/*
 * Filename    : arch/mips/vr41xx/common/power.c
 * Author      : Jackal Chuang
 * Email       : ct_chuang@coventive.com
 * Create Date : Wed Step 19 2002
 * 
 * Modify Date : Wed Dec 4 2002  
 *               
 * Description : - implement "standby,suspend,sleep" and "restart" funciton
 *                 on VR4121 + Road MR-2A platform
 *               - base on Michael Klar's power.c source.
 * Source      : Copyright (c) 2002 Coventive
 * 
 * Change      : - (20030415) 2.4.10 update to 2.4.18
 * 
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 */

#include <linux/init.h>
#include <linux/pm.h>
#include <linux/sysctl.h>
#include <linux/kernel.h>
#include <linux/delay.h>
#include <linux/mm.h>
#include <linux/acpi.h>

#include <asm/vr41xx.h>
#include <asm/io.h>

/*********************************************************
 * define
 *********************************************************/
#define ACPI_VERSION	"2003-05-30 V1.4"

/* 2003-04-06 V1.3 -> 2003-05-30 V1.4:
 * Fix the suspend loop will cause system die
 * 1. remove the timer_init after suspend mode
 * 2. add disable INT7 before suspend and enable INT7 after suspend
 */

#undef DEBUG_LED

/*********************************************************
 * function prototype
 *********************************************************/
extern struct semaphore vr41xx_dma_sem;

//extern void sound_PHL4000_barcode_off(void);

#ifdef CONFIG_PM_STANDBY
void PM_do_standby(void);
#endif

#ifdef CONFIG_PM_POWERED_SUSPEND
void PM_do_suspend(void);
#endif

#ifdef CONFIG_PM_SUSPEND_WAKEUP
void vr41xx_hibernate(void);
#endif

/* extern function */
extern unsigned long get_mr2a_rtc_time(void);
extern void pmu_timer_lowpower(int);
extern int pmu_pwoff_status(void);
extern int pm_access_device(void);

#ifdef CONFIG_PM_STANDBY
static inline void vr41xx_standby(void)
{
   asm volatile (
		 " .set noreorder\n"
		 " .align 4\n"
		 " standby\n"
		 " nop\n"
		 " nop\n"
		 " nop\n"
		 " .set reorder\n"
		 );
}

static int pm_do_standby(ctl_table *ctl, 
			 int write, 
			 struct file *file,
			 void *buffer, 
			 size_t *len)
{
   if (!write) {
      if (file->f_pos) {
	 *len = 0;
	 return 0;
      }
   } else {
      pmu_timer_lowpower(1);
   }
   file->f_pos += *len;
   
   return 0;
}


/*
 * Jackal Note: ( 20021206 )
 * CPU             : VR4121 (standby mode)
 * Wakeup          : all interrupts
 * Redefine wakeup : - press PWSW < 300 ms.
 *                   - keyboard
 *                   - touch panel
 *                   - barcode
 * Reboot          : - RSTSW
 * Sleep           : - battery low
 *                   - battery cover open
 */          
void PM_do_standby(void)
{
   int i;
   static unsigned short irq_mask_probe[(VR41XX_IRQ_MAX + 9)/16];
   
   /* (MR2A) VEXTIN(cradle charging) PMU_INT_ST_CLR ; ack interrupt */
   *(unsigned short *)0xaa0002b0 &= ~0x0002;
   /* (MR2A) mask VEXTIN(cradle charging) interrupt */ 
   *(unsigned short *)0xaa0002b2 &= ~0x0002;
  
   irq_mask_probe[0] = read_32bit_cp0_register(CP0_STATUS);
   irq_mask_probe[1] = *VR41XX_MSYSINT1REG;
   irq_mask_probe[2] = *VR41XX_MSYSINT2REG;
   irq_mask_probe[3] = *VR41XX_MGIUINTLREG;
   irq_mask_probe[4] = *VR41XX_MGIUINTHREG;
   
   /*  
    * supply interrupt lists :
    * 
    *  2 = IP2 - All other interrupts
    * 
    *  8 = VR41XX_IRQ_BATTERY   (Sleep/Reboot)
    *  9 = VR41XX_IRQ_POWER     (Wakeup)
    * 11 = VR41XX_IRQ_ETIMER   
    * 13 = VR41XX_IRQ_PIU       (Wakeup)
    * 15 = VR41XX_IRQ_KIU       (Wakeup)
    * 16 = VR41XX_IRQ_GIU
    * 
    * 40 = GPIO0  (INT)
    * 60 = GPIO10 (TRG SW Input) (Wakeup)
    * 	Barcode (press)
    */

   for (i = VR41XX_IRQ_MAX; i > 0; i--){
      if (i != VR41XX_IRQ_TIMER && (irq_mask_probe[(i + 8)/16] & ((unsigned short)1 << ((i + 8) & 15))))
	if (i != 2  &&
	    i != 8  && i != 9 && i != 11 && i != 13 && i != 15 && i != 16 &&
	    i != 40 && i != 60)
	  disable_irq(i);
   }
   
   disable_irq(VR41XX_IRQ_TIMER);
   /*-----------------------------------------------*/
   vr41xx_standby();
   /*-----------------------------------------------*/
   enable_irq(VR41XX_IRQ_TIMER);
   
   for (i = VR41XX_IRQ_MAX; i > 0; i--){
      if (i != VR41XX_IRQ_TIMER && (irq_mask_probe[(i + 8)/16] & ((unsigned short)1 << ((i + 8) & 15))))
	if (i != 2  &&
	    i != 8  && i != 9 && i != 11 && i != 13 && i != 15 && i != 16 &&
	    i != 40 && i != 60)
	  enable_irq(i);
   }
   
   /* (MR2A) VEXTIN(cradle charging) PMU_INT_ST_CLR ; ack interrupt */   
   *(unsigned short *)0xaa0002b0 &= ~0x0002;
   /* (MR2A) supply VERTIN(cradle charging) interrupt */
   *(unsigned short *)0xaa0002b2 |=  0x0002;
   
   xtime.tv_sec = get_mr2a_rtc_time();
   xtime.tv_usec = 0;
}
#endif // CONFIG_PM_STANDBY

#ifdef CONFIG_PM_POWERED_SUSPEND

/*
 * pm_register list 
 */

/* (1) arch/mips/vr41xx/vr4121/roadht/kbd.c */
static int pm_kbd_rqst(int rqst)
{
   static unsigned short save_regs[4];

   switch (rqst) {
    case PM_SUSPEND:
      // Save hardware registers.
      save_regs[0] = *VR41XX_SCANLINE;
      save_regs[1] = *VR41XX_KIUSCANREP;
      save_regs[2] = *VR41XX_KIUWKS;
      save_regs[3] = *VR41XX_KIUWKI;
      
      *VR41XX_MKIUINTREG = VR41XX_KIUINT_SCANINT;
      break;
    case PM_RESUME:
      *VR41XX_SCANLINE =   save_regs[0];
      *VR41XX_KIUSCANREP = save_regs[1];
      *VR41XX_KIUWKS =     save_regs[2];
      *VR41XX_KIUWKI =     save_regs[3];
      
      *VR41XX_MKIUINTREG = VR41XX_KIUINT_KDATRDY | VR41XX_KIUINT_KDATLOST;
      break;
   }
   return 0;
}


/* (3) drivers/char/console.c */
extern void unblank_screen(void);
extern void do_blank_screen(int);

static int pm_con_rqst(int rqst)
{
	switch (rqst)
	{
	case PM_RESUME:
		unblank_screen();
		break;
	case PM_SUSPEND:
		do_blank_screen(0);
		break;
	}
	return 0;
}

/* (4) drivers/char/tpanel.c */
static int pm_tpanel_rqst(int rqst)
{
   static unsigned short piu_state[4];
                   
   switch (rqst) {
    case PM_SUSPEND:
      piu_state[0] = *VR41XX_PIUCNTREG;
      piu_state[1] = *VR41XX_PIUSIVLREG;
      piu_state[2] = *VR41XX_PIUSTBLREG;
      piu_state[3] = *VR41XX_MPIUINTREG;
      break;
    case PM_RESUME:
      *VR41XX_PIUCNTREG = piu_state[0];
      *VR41XX_PIUSIVLREG = piu_state[1];
      *VR41XX_PIUSTBLREG = piu_state[2];
      *VR41XX_PIUINTREG = ~0;
      barrier();
      *VR41XX_MPIUINTREG = piu_state[3];
      break;
   }
   
   return 0;
}

/* (5) drivers/video/fbcon.c */
extern void pm_fbcon_rqst(int);
  
/* (6) drivers/mtd.new/mtdcore.c */
extern void pm_mtd_rqst(int);

static inline void vr41xx_suspend(void)
{
#ifdef DEBUG_LED 
   *(unsigned short *)0xab000100 |=  0x3000;
   *(unsigned short *)0xab000104 &= ~0x2000;
   *(unsigned short *)0xab000104 |=  0x1000;
#endif
   
#if 1 /* asm example */
   asm volatile(
		".set\tnoreorder\n\t"
		".align 4\n\t"
		/* Make sure DRAM go into self-refresh */
		"lui\t$8,0xab00\n\t"
		"lhu\t$9,0x000e($8)\n\t"
		"sh\t$9,0x000e($8)\n\t"
		/* Issue Three dummy read */
		"lhu\t$9,0x0012($8)\n\t"
		"lhu\t$9,0x0012($8)\n\t"
		"lhu\t$9,0x0012($8)\n\t"
		/* Enable IE bit */
		"mfc0\t$8,"STR(CP0_STATUS)"\n\t"
	        "li\t$9,0x00000001\n"
	        "or\t$8,$9\n"
	        "mtc0\t$8,"STR(CP0_STATUS)"\n\t"
		/* Into suspend mode */
		"suspend\n\t"
		"nop\n\t"
   );
#else /* asm + c */		

   {
	u16 bctctlreg;
	int i;
	
	bcuctlreg = *(unsigned short *)0xab00000e;
	*(unsigned short *)0xab00000e = bcuctlreg;
	i=3;
	while(i--)
		bcuctlreg = *(unsigned short *)0xab000012;
	
	status = read_32bit_cp0_register(CP0_STATUS);
	write_32bit_cp0_register(CP0_STATUS, status | ST0_IE);
   }
   
   asm volatile(
		".set noreorder\n"
		".align 4\n"
		"suspend\n"
		"nop\n"
		"nop\n"
		"nop\n"
		".set reorder\n"
   );
#endif
   
#ifdef DEBUG_LED
   *(unsigned short *)0xab000100 |=  0x1000;
   *(unsigned short *)0xab000104 &= ~0x1000;
#endif
}

static int pm_do_suspend(ctl_table *ctl,
			 int write,
			 struct file *file,
			 void *buffer,
			 size_t *len)
{
   if (!write){
      if (file->f_pos){
	 *len = 0;
	 return 0;
      }
   }else{
      PM_do_suspend();
      //pmu_timer_lowpower(2);
   }
   file->f_pos += *len;
   
   return 0;
}

extern void pm_timer_init(void);
/*
 * Jackal Note: ( 20021206 )
 * CPU             : VR4121 ( suspend )
 * Wakeup          : power,rstsw,etimer,rtclong1,rtclong2,keytouch,pentouch
 *                   gpio(14:9),gpio(3:0),DCD#(gpio,siu),battintr
 * Redefine wakeup : - press PWSW < 300 ms.
 *                   - keyboard
 *                   - touch panel
 * Reboot          : - RSTSW
 * Sleep           : - battery low
 *                   - battery cover open
 */
void PM_do_suspend(void)
{
   int i;
   unsigned short kiuscans_sstat;
   unsigned short do_suspend = 0;
   unsigned short padstate = 0;
   static unsigned short irq_mask_probe[(VR41XX_IRQ_MAX + 9)/16];
   static unsigned short kiuscanrep_atscan;
   
   unsigned long flags, status;

   /*********** Disable IE bit , 2003-0625 ***********/
#if 1
   status = read_32bit_cp0_register(CP0_STATUS);
   write_32bit_cp0_register(CP0_STATUS, status & ~ST0_IE);
#endif
   /**************************************************/
   
   /* (MR2A) VEXTIN(cradle charging) PMU_INT_ST_CLR ; ack interrupt */
   *(unsigned short *)0xaa0002b0 &= ~0x0002;
   /* (MR2A) mask VEXTIN(cradle charging) interrupt */ 
   *(unsigned short *)0xaa0002b2 &= ~0x0002;
   
   irq_mask_probe[0] = read_32bit_cp0_register(CP0_STATUS);
   irq_mask_probe[1] = *VR41XX_MSYSINT1REG;
   irq_mask_probe[2] = *VR41XX_MSYSINT2REG;
   irq_mask_probe[3] = *VR41XX_MGIUINTLREG;
   irq_mask_probe[4] = *VR41XX_MGIUINTHREG;
   
   pm_kbd_rqst(PM_SUSPEND);
   pm_con_rqst(PM_SUSPEND);
   pm_tpanel_rqst(PM_SUSPEND);
   
   /*  
    * supply interrupt lists :
    * 
    *  2 = IP2 - All other interrupts
    * 
    *  8 = VR41XX_IRQ_BATTERY   (Sleep/Reboot)
    *  9 = VR41XX_IRQ_POWER     (Wakeup)
    * 11 = VR41XX_IRQ_ETIMER   
    * 13 = VR41XX_IRQ_PIU       (Wakeup)
    * 15 = VR41XX_IRQ_KIU       (Wakeup)
    */
   for (i = VR41XX_IRQ_MAX; i > 0; i--){
      if (i != VR41XX_IRQ_TIMER && (irq_mask_probe[(i + 8)/16] & ((unsigned short)1 << ((i + 8) & 15))))
	if (i != 2  &&
	    i != 8  && i != 9 && i != 11 && i != 13 && i != 15)
	  disable_irq(i);
   }
   
   /*=========================================================*/   
   /* 
    * Check PIU mode 
    */
   
   /* enable suspend touchpanel wakeup interrupt -> suggest from OPTO*/
   *(unsigned short *)0xab00008c |= (1 << 13);

   padstate = *(unsigned short *)0xab000122;
   padstate = (padstate >> 10) & 0x0007;
   
   /*
    * function : udelay(int) unit: micro second
    */
   switch(padstate) {
    case 4 : /* WaitPenTouch state (12:10) 100 */
      /* PIUSEQEN = 0 */
      *(unsigned short *)0xab000122 &= ~0x0004;
      udelay(5); /* waiting I/O */
      /* PIUPWR   = 1 */
      *(unsigned short *)0xab000122 |= 0x0002;
      udelay(5);
    case 1 : /* Standby state (12:10) 001 */
      /* PIUPWR   = 0 */
      *(unsigned short *)0xab000122 &= ~0x0002;
      udelay(5);
      /* PIUPWR   = 0 */
      *(unsigned short *)0xab000122 &= ~0x0002;
      udelay(5);
    case 0 : /* Disable state (12:10) 000 */
      break;
    default :
      do_suspend++;
      break;
   }
   /* wating STABLE counter */
   udelay(30*0x3f);
   /*=========================================================*/

   /*=========================================================*/   
   /* 
    * Check KIU mode 
    */
   kiuscanrep_atscan = (*(unsigned short *)0xab000190 & 0x0001);
   /* confirm KIUSCANS(SSTAT) whether "waitkeyin" state*/
   kiuscans_sstat = (*(unsigned short *)0xab000192 & 0x0003);
   
   /* KIUSCANREP -- auto scan mode */
   *(unsigned short *)0xab000190 |= 0x0001;
   if (kiuscans_sstat != 0x0001) do_suspend++;
   /*=========================================================*/
  
   if (!do_suspend) {
      disable_irq(7); /* disable irq = 3 */
      vr41xx_suspend();
      enable_irq(7);
   }
   
   /*=========================================================*/   
   /*
    * Resume 
    */

   /*********** Enable IE bit , 2003-0625 ************/
   status = read_32bit_cp0_register(CP0_STATUS);
   write_32bit_cp0_register(CP0_STATUS, status | ST0_IE);
   /**************************************************/
   
   /* (R1) KIU */
   *(unsigned short *)0xab000190 |= kiuscanrep_atscan;
   
   /* (R2) PIU */
   /* PIUPWR = 1 */
   *(unsigned short *)0xab000122 |=  0x0002;
   /* PIUMODE(1:0) = 00 */
   *(unsigned short *)0xab000122 &= ~0x0018;
   /* PADATSCAN = 1 */
   *(unsigned short *)0xab000122 |=  0x0102;
   /* PADATSTOP = 1 */
   *(unsigned short *)0xab000122 |=  0x0202;
   /* PIUSEQEN = 1 */
   *(unsigned short *)0xab000122 |=  0x0004;

   for (i = VR41XX_IRQ_MAX; i > 0; i--){
      if (i != VR41XX_IRQ_TIMER && (irq_mask_probe[(i + 8)/16] & ((unsigned short)1 << ((i + 8) & 15))))
	if (i != 2  &&
	    i != 8  && i != 9 && i != 11 && i != 13 && i != 15)
	  enable_irq(i);
   }
   
   save_and_cli(flags);
   pm_kbd_rqst(PM_RESUME);
   pm_con_rqst(PM_RESUME);
   pm_tpanel_rqst(PM_RESUME);
   restore_flags(flags);
   
   *(unsigned short *)0xaa0002b0 &= ~0x0002;
   *(unsigned short *)0xaa0002b2 |=  0x0002;

   xtime.tv_sec = get_mr2a_rtc_time();
   xtime.tv_usec = 0;
   
   /*=========================================================*/   
   do_suspend = 0;
}
#endif // CONFIG_PM_POWERED_SUSPEND 

#ifdef CONFIG_PM_SUSPEND_WAKEUP
void vr41xx_hibernate(void)
{
#if 1
   pmu_set_poweroff();
#else
   asm volatile(
		" .set noreorder\n"
		" .align 4\n"
		" hibernate\n"
		" nop\n"
		" nop\n"
		" nop\n"
		" .set reorder\n"
		);
#endif
}

static int pm_do_hibernate(ctl_table *ctl,
		       int write,
		       struct file *file,
		       void *buffer,
		       size_t *len)
{
   
   if (!write){
      if (file->f_pos){
	 *len = 0;
	 return 0;
      }
   }
   else{
      /* No implement */
      vr41xx_hibernate();
   }
   
   file->f_pos += *len;
   
   return 0;
}


#endif

static struct ctl_table pm_table[] =
{
#ifdef CONFIG_PM_SUSPEND_WAKEUP
     {ACPI_SLEEP, "hibernate", NULL, 0, 0600, NULL, &pm_do_hibernate},
#endif
   
#ifdef CONFIG_PM_POWERED_SUSPEND
     {ACPI_S1_SLP_TYP, "suspend", NULL, 0, 0600, NULL, &pm_do_suspend},
#endif
   
#ifdef CONFIG_PM_STANDBY
     {ACPI_S0_SLP_TYP, "standby", NULL, 0, 0600, NULL, &pm_do_standby},
#endif
     {0}
};

static struct ctl_table pm_dir_table[] =
{
     {CTL_ACPI, "pm", NULL, 0, 0555, pm_table},
     {0}
};

/*
 * Initialize the power management user interface
 */
static int __init pm_init(void)
{
   printk(KERN_INFO "ACPI Controller Driver v" ACPI_VERSION "\n");
   register_sysctl_table(pm_dir_table, 1);
   
   return 0;
}

__initcall(pm_init);

