/*
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 *
 * Copyright (C) 1997, 2001 Ralf Baechle
 * Copyright 2001 MontaVista Software Inc.
 * Author: jsun@mvista.com or jsun@junsun.net
 */
#include <linux/sched.h>
#include <linux/mm.h>
#include <asm/io.h>
#include <asm/pgtable.h>
#include <asm/processor.h>
#include <asm/reboot.h>
#include <asm/system.h>
#ifdef CONFIG_ROAD_HT
#include <asm/vr41xx.h>
#endif

void vr41xx_restart(char *command)
{
#ifdef CONFIG_ROAD_HT
   unsigned long cpu_version = read_32bit_cp0_register(CP0_PRID);
   unsigned long config = read_32bit_cp0_register(CP0_CONFIG);
   int i;
   
   /* stop RTCLong1 register */
   *(volatile short *)VR41XX_RTCL1LREG = 0;
   *(volatile short *)VR41XX_RTCL1HREG = 0;
   
   *(volatile short *)VR41XX_RTCL2LREG = 0;
   *(volatile short *)VR41XX_RTCL2HREG = 0;
   
   *(volatile short *)VR41XX_RTCINTREG = 0xf;
   
   *(volatile short *)VR41XX_MSYSINT1REG = 0;
   *(volatile short *)VR41XX_MSYSINT2REG = 0;
   
   change_cp0_status((ST0_BEV | ST0_ERL), (ST0_BEV | ST0_ERL));
   
   if(cpu_version >= 0xc83){          
      change_cp0_config(CONF_CM_CMASK, CONF_CM_CACHABLE_NONCOHERENT);
   } else {          
      config  = config | 0x00400000;
      write_32bit_cp0_register(CP0_CONFIG,config);
   }  
   flush_cache_all();
   write_32bit_cp0_register(CP0_WIRED, 0);
   
   /* disable interrupt */
   for (i = 0; i <= 71; i ++) disable_irq(i);
   
   *(unsigned short *)(0xaa0002a0) = 0;
   *(unsigned short *)(0xaa0002ae) = 0;
   *(unsigned short *)(0xaa0002b2) = 0;
   
   #if 1
   /* Jackal add : clear CFI command */
     { 
	u16 *base = (u16 *)0xbe000000;
	
	*VR41XX_BCUCNTREG1 |= 0x0050; /* Flash Write Enable */
	*VR41XX_GIUPIODL |= 0x0800;   /* Flash ROM VPEN Enable */
	
	while( (*VR41XX_GIUPIODL & 0x0400 ) != 0x0400 );  /* FLash ROM Status */
	
	*base = 0x0050;

	*VR41XX_BCUCNTREG1 &= ~0x0050;
     }
   
   
   /* ORANGE */
   *((unsigned short*)(0xab000100)) |= 0x3000;
   *((unsigned short*)(0xab000104)) |= 0x3000;
   //mdelay(1000);
   
   __asm__ __volatile__("jr\t%0;nop;nop"::"r"(0xbfc00000));
   #else						      
   *(unsigned short *)0xaa000000 = 0x0001;	
   /* Reset Setting */
   *(unsigned short *)( 0xaa000098 ) = 0x0000;        /* TCNT1 */
   *(unsigned short *)( 0xaa00009A ) = 0xFFFF;        /* TLD1  */
   *(unsigned short *)( 0xaa00009C ) = 0x0005;        /* TRGA1 */
   *(unsigned short *)( 0xaa00009E ) = 0x0020;        /* TRGB1 */
   *(unsigned short *)( 0xaa0000A2 ) = 0xE440;        /* TCON1 */
   *(unsigned short *)( 0xaa0000A4 ) = 0x1000;        /* TSEL1 */
   *(unsigned short *)( 0xaa0000A6 ) = 0xC000;        /* TCAL1 */ 
   #if 1
   /* ORANGE */
   *((unsigned short*)(0xab000100)) |= 0x3000;
   *((unsigned short*)(0xab000104)) |= 0x3000;
   #endif
   mdelay(1000);
   
   /* Reset Start */
   *(unsigned short *)( 0xaa000080 ) = 0x0002;        /* TMOD  */
   udelay(2000);	
   #endif /*----- #if 1 -----*/
#else
   change_cp0_status((ST0_BEV | ST0_ERL), (ST0_BEV | ST0_ERL));
   change_cp0_config(CONF_CM_CMASK, CONF_CM_UNCACHED);
   flush_cache_all();
   write_32bit_cp0_register(CP0_WIRED, 0);
   __asm__ __volatile__("jr\t%0"::"r"(0xbfc00000));
#endif
}

void vr41xx_halt(void)
{
   printk(KERN_NOTICE "\n** You can safely turn off the power\n");
   while (1);
}

void vr41xx_power_off(void)
{
   vr41xx_halt();
}
