/*
 * Filename    : arch/mips/vr41xx/common/sleep.c
 * Author      : Jackal Chuang
 * Email       : ct_chuang@coventive.com
 * Create Date : Wed Step 19 2002
 * 
 * Modify Date : Wed Dec 4 2002  Thu Dec 12 2002
 *               
 * Description : - implement "sleep" funciton on VR4121 + Road MR-2A platform
 * Source      : Copyright (c) 2002 Coventive
 * 
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * 2003-06-03 modify:
 * 	- add turn off buzzer function before sleep and after sleep
 * 	- add turn off led function after sleep
 */
#include <linux/init.h>
#include <linux/pm.h>
#include <linux/malloc.h>

#include <linux/kernel.h>
#include <linux/config.h>
#include <linux/delay.h>
#include <linux/mm.h>
#include <linux/irq.h> 
#include <linux/serial_reg.h>

#include <asm/vr41xx.h>
#include <asm/io.h>

#include "sleep_offset.h"

extern void save_regs(void);
extern void wakeup_restore_regs(void);
extern unsigned long *save_regs_arr;
extern unsigned long get_mr2a_rtc_time(void);
extern int pm_access_device(void);
extern int mips_counter_frequency;
extern void set_freq(int);		/* 2003-0530 */

/*
 * peripheral initial
 * 
 * 1)timer
 * 2)keyboard
 * 3)pcmcia
 * 
 */
//static void pm_timer_init(void)	/* Jackie */
void pm_timer_init(void)
{
   u32 count,flags;
   u16 newstatus;
   
   save_and_cli(flags);
   newstatus = read_32bit_cp0_register(CP0_STATUS);
   newstatus |= ((unsigned short)1 << (7 + 8));
   set_cp0_status(ST0_IM & newstatus);
   restore_flags(flags);
   
   xtime.tv_sec = get_mr2a_rtc_time();
   xtime.tv_usec = 0;
   
   count = read_32bit_cp0_register(CP0_COUNT);
   write_32bit_cp0_register (CP0_COMPARE, count + (mips_counter_frequency / HZ));
}

static void pm_kbd_init(unsigned short kiu_scanline)
{
   // turn on the KIU clock
   *VR41XX_CMUCLKMSK |= 0x0008;
   // reset the KIU
   *VR41XX_KIURST = 0x0001;
   barrier();
   
   *VR41XX_SCANLINE = kiu_scanline;
   // enable scan, auto scan, auto stop, and one stop rep
   *VR41XX_KIUSCANREP = VR41XX_KIUSCANREP_KEYEN | VR41XX_KIUSCANREP_ATSCAN | VR41XX_KIUSCANREP_ATSTP | (1 << 4);
   *VR41XX_KIUWKS = (12-1) | ((12-1) << 5) | ((12-1) << 10);
   *VR41XX_KIUWKI = 0x03ff;
   barrier();
   
   // clear any pending interrupts
   *VR41XX_KIUINT = 0x0007;
   barrier();
   
   // unmask the keyboard interrupts that we are interested in
   *VR41XX_MKIUINTREG = VR41XX_KIUINT_KDATRDY | VR41XX_KIUINT_KDATLOST;
}

#if 0
static void pm_pcmcia_init(void)
{
   #if (CONFIG_PHL4000 || CONFIG_PHL4100)
   *(unsigned short *)(0xaa00005c)  =  0x0000;
   *(unsigned short *)(0xaa000040) |=  0x0002;
   *(unsigned short *)(0xaa000044) &= ~0x0002;
   #else /* on bb board */
   /* GPIO19 for CS# */
   *(unsigned short *)(VR41XX_GIUIOSELH)   |=  0x0008 ;
   *(unsigned short *)(VR41XX_GIUPIODH)    &= ~0x0008 ;
   *(unsigned short *)(0xaa000040)  &= ~0x0001 ;       
   #endif 
   
   #if 0
   *(volatile unsigned short *)(VR41XX_GIUIOSELH) |= 0x0002 ;
   barrier();
   *(volatile unsigned short *)(VR41XX_GIUPIODH)  |= 0x0002 ;
   barrier();
   *(volatile unsigned short *)(VR41XX_GIUIOSELH) &= ~0x0002;
   barrier();
   #endif
   
   #if 0
   *(unsigned short *)( VR41XX_GIUINTTYPH ) |= 0x0002 ;
   barrier();
   *(unsigned short *)( VR41XX_GIUINTENH ) |= 0x0002 ;
   barrier();
   *(unsigned short *)( VR41XX_GIUINTHTSELH ) |= 0x0002 ;
   barrier();
   *(unsigned short *)( VR41XX_MGIUINTHREG ) |= 0x0002 ;
   barrier();
   *(unsigned short *)( VR41XX_MSYSINT1REG ) |= 0x0503;
   barrier();
   *(unsigned short *)( VR41XX_GIUINTSTATH ) |= 0x0002 ;
   barrier();
   #endif
   
   *(volatile unsigned char *)( 0xb40003E0 ) = 0x2f;
   barrier();
   //*(volatile unsigned char *)( 0xb40003E1 ) = 0x0b;
   barrier();
}
#endif

/*
 * pm_callback
 * 1)serial
 * 2)timer
 */
static void pm_serial(int rqst)
{
   static unsigned char save_state[7];
   unsigned char dummy;
   unsigned long flags;

   switch (rqst) {
    case PM_SUSPEND:
      save_and_cli(flags);
      
      /* wait for anything in the xmit FIFO to send */      
      while (!(*VR41XX_SIULS & UART_LSR_TEMT))	barrier();
      save_state[0] = *VR41XX_SIULC;
      *VR41XX_SIULC = UART_LCR_DLAB;	/* DLAB = 1 to read baud rate */
      barrier();
      
      save_state[1] = *VR41XX_SIUDLL;
      save_state[2] = *VR41XX_SIUDLM;
      barrier();
      
      *VR41XX_SIULC = 0;
      barrier();
      
      save_state[3] = *VR41XX_SIUIE;
      *VR41XX_SIUIE = save_state[3] & ~UART_IER_THRI;
      save_state[4] = *VR41XX_SIUMC;
      save_state[5] = *VR41XX_SIUIRSEL;
      save_state[6] = *VR41XX_SIUFC;
      
      restore_flags(flags);
      break;
      
    case PM_RESUME:
      save_and_cli(flags);
      
      *VR41XX_GIUIOSELL |= 0x04;
      *VR41XX_GIUPIODL &= 0xfffb;
      *VR41XX_GIUPIODL |= 0x0004;
      /* reset SIU */
      *VR41XX_SIURESET = 0x01;
      udelay(100);
      *VR41XX_SIURESET = 0x00;
      *VR41XX_SIUIRSEL = 0x01;
      *VR41XX_SIUCSEL  = 0x01;
      
      *VR41XX_SIUFC = UART_FCR_CLEAR_RCVR | UART_FCR_CLEAR_XMIT;
      dummy = *VR41XX_SIURB;
      dummy = *VR41XX_SIUMS;
      dummy = *VR41XX_SIUIID;
      barrier();
      while ( *VR41XX_SIULS & UART_LSR_DR )
	dummy = *VR41XX_SIURB;
      
      *VR41XX_SIULC = UART_LCR_DLAB;
      barrier();
      *VR41XX_SIUDLL = save_state[1];
      *VR41XX_SIUDLM = save_state[2];
      barrier();
      *VR41XX_SIULC = 0;
      barrier();
      *VR41XX_SIUIE = save_state[3];
      *VR41XX_SIUMC = save_state[4];
      *VR41XX_SIUIRSEL = save_state[5];
      barrier();
      *VR41XX_SIULC = save_state[0];
      *VR41XX_SIUFC = save_state[6];
      
      restore_flags(flags);      
      break;
   }
}

extern rwlock_t xtime_lock;

static int pm_timer(int rqst)
{
   
   unsigned long flags;
   u32 count;
   
   switch (rqst){
    case PM_SUSPEND:
      disable_irq(VR41XX_IRQ_TIMER);
      break;
    case PM_RESUME:
      write_lock_irqsave(&xtime_lock, flags);
      enable_irq(VR41XX_IRQ_TIMER);
      
      xtime.tv_sec = get_mr2a_rtc_time();
      xtime.tv_usec = 0;
      
      count = read_32bit_cp0_register(CP0_COUNT);
      write_32bit_cp0_register (CP0_COMPARE, count + (mips_counter_frequency / HZ));
      
      write_unlock_irqrestore(&xtime_lock, flags);
      break;
   }
   
   return 0;
}

/*************************************************************
 * sleep  : 
 *        - save BCU,CMU,ICU,PMU,GIU register.
 *        - save mr2a register.
 *        - save gpr and cp0 register.
 *        - shut down vr4121 cpu.
 * wakeup :
 *        - restore gpr, cp0, mr2a
 *        - restore BCU,CMU,ICU,PMU,GIU register.
 *        - initial timer.
 *        - initial barcode.     -- extern.
 *        - initial keyboard.
 *        - initial pcmcia.
 *        - initial sed1376(lcd) -- extern.
 *************************************************************/
int PM_do_sleep(void)
{
   int i;
   unsigned long  k;
   unsigned long  resume_addr;
   unsigned short sleep_flag;
   
   static unsigned short bcuregs[4] ,cmuclkmsk;
   static unsigned short icuregs[11],pmuregs[6],giuregs[18];
   static unsigned short kiuregs;

#ifdef CONFIG_BAR_CODE   
   extern void pm_barcode_init(void);
#endif
#ifdef CONFIG_ZEEVO_BLT
   extern void pm_bluetooth_init(void);
#endif 
   extern void pm_lcd_callback(int);
   extern void pm_sed1376_init(void);
   extern void pm_lcd_off(void);
   extern void pm_lcd_on(void);
   
   pm_lcd_off();
   set_freq(0);	/* 2003-0530 */
   /* turn off LED */
   *(unsigned short *)0xab000100 |=  0x3000;
   *(unsigned short *)0xab000104 &= ~0x3000;
   
   save_regs_arr = kmalloc(SR_SIZE*sizeof(long),GFP_ATOMIC);
   if (!save_regs_arr) return -ENOMEM;
   resume_addr = virt_to_phys(wakeup_restore_regs) + KSEG0;
   
   /* save sleep_flag  to MR-2A PGUBALREG */
   *(unsigned short *)0xaa000000 = 0xcccc;
   /* save resume_addr to MR-2A PGUALREG and PGUAHREG */
   *(unsigned long *)0xaa000004 =  resume_addr;
   
//   pm_pcmcia(PM_SUSPEND);
   
   pm_serial(PM_SUSPEND);
   pm_send_all(PM_SUSPEND, (void *)2);
   pm_timer(PM_SUSPEND);
   
   cli();
   /*=======================================================
    * save peripheral register
    *=======================================================*/
   /* (1) VR4121 BCU */
   k = 0x0b000000 + KSEG1;
   bcuregs[0] = *(unsigned short *)(k+0x0000);
   bcuregs[1] = *(unsigned short *)(k+0x000a);
   bcuregs[2] = *(unsigned short *)(k+0x000e);   
   bcuregs[3] = *(unsigned short *)(k+0x001e);
   /* (2) VR4121 CMU */
   cmuclkmsk   = *VR41XX_CMUCLKMSK;
   /* (3) VR4121 ICU */
   k = 0x0b00008c + KSEG1;
   for(i=0; i<8; i++){
      icuregs[i] = *(unsigned short *)(k + (i*2));
   }
   k = 0x0b000000 + KSEG1;
   icuregs[i]   = *(unsigned short *)(k+0x0206);
   icuregs[i+1] = *(unsigned short *)(k+0x0208);
   icuregs[i+2] = *(unsigned short *)(k+0x020a);
   /* (4) VR4121 PMU */
   k = 0x0b000000 + KSEG1;   
   pmuregs[0] = *(unsigned short *)(k+0x00a0);
   pmuregs[1] = *(unsigned short *)(k+0x00a2);
   pmuregs[2] = *(unsigned short *)(k+0x00a4);
   pmuregs[3] = *(unsigned short *)(k+0x00a6);
   pmuregs[4] = *(unsigned short *)(k+0x00a8);
   pmuregs[5] = *(unsigned short *)(k+0x00ac);
   /* (6) VR4121 GIU */
   k = 0x0b000100 + KSEG1;   
   for(i = 0; i < 16; i++){
      giuregs[i] = *(unsigned short *)(k + (i*2));
   }
   k=0x0b000000+KSEG1;
   giuregs[i] = *(unsigned short *)(k+0x02e0);
   giuregs[i+1] = *(unsigned short *)(k+0x02e2);
   /* (7) KIU */
   k = 0x0b000000 + KSEG1;
   kiuregs = *(unsigned short *)(k + 0x019e);
   /*===========================================================*/
   /*
    * save GPR[0..31],HI,LO, CP0_REG[0..31]
    */
   save_regs();
   
   //flush_cache_all();
   //flush_tlb_all();
  
   sleep_flag = *(unsigned short *)0xaa000000;

   if ( sleep_flag == 0xcccc ){
      
      flush_cache_all();
      flush_tlb_all();
      
      pm_lcd_callback(1);
      *(unsigned short *)0xaa000044 &= 0x001f;
      udelay(10);
      
      /* vr4121 gpio 0  - 15 */
      *(unsigned short *)0xaa000104 &= 0x00f0;
      /* vr4121 gpio 16 - 31 */
      *(unsigned short *)0xaa000106 &= 0x0000;
      /* mask vr4121_cmuclkmsk */
      *VR41XX_CMUCLKMSK ^= cmuclkmsk;

      /* MR-2A CS singal select */
      *(unsigned short *)0xaa00005c &= 0x0000;
      /* MR-2A => gpio 0 - 7    */
      *(unsigned short *)0xaa000044 &= 0x0000;
      /* MR-2A => cmuclkmsk register */      
      *(unsigned short *)0xAA000260 &= ~0x4423;

      /* MR-2A => turn off cpu */
      *(unsigned short *)0xaa000282 |= 0x0001;
      *(unsigned short *)0xaa000282 |= 0x0001;
   }
   /*===========================================================*/
   /* reset peripheral */
   /* (1) reset piu */
   *VR41XX_PIUCNTREG = 0x0001;
   
   /* (1) VR4121 BCU */
   k = 0x0b000000 + KSEG1;
   *(unsigned short *)(k+0x0000) = bcuregs[0];
   *(unsigned short *)(k+0x000a) = bcuregs[1];
   *(unsigned short *)(k+0x000e) = bcuregs[2];
   *(unsigned short *)(k+0x001e) = bcuregs[3];
   /* (2) VR4121 CMU */
   *VR41XX_CMUCLKMSK   = cmuclkmsk;
   /* (3) VR4121 ICU */
   k = 0x0b00008c + KSEG1;
   for(i=0;i<8;i++){
      *(unsigned short *)(k + i*2) = icuregs[i];
   }
   k = 0x0b000000 + KSEG1;
   *(unsigned short *)(k+0x0206) = icuregs[i];
   *(unsigned short *)(k+0x0208) = icuregs[i+1];
   *(unsigned short *)(k+0x020a) = icuregs[i+2];
   /* (4) VR4121 PMU */
   k = 0x0b000000 + KSEG1;   
   *(unsigned short *)(k+0x00a0) = pmuregs[0];
   *(unsigned short *)(k+0x00a2) = pmuregs[1];
   *(unsigned short *)(k+0x00a4) = pmuregs[2];
   *(unsigned short *)(k+0x00a6) = pmuregs[3];
   *(unsigned short *)(k+0x00a8) = pmuregs[4];
   *(unsigned short *)(k+0x00ac) = pmuregs[5];
   
   /* (6) VR4121 GIU */
   k = 0x0b000100 + KSEG1;   
   for(i=0;i<16;i++){
      *(unsigned short *)(k + i*2)=giuregs[i];
   }
   k=0x0b000000+KSEG1;
   *(unsigned short *)(k+0x02e0) = giuregs[i];
   *(unsigned short *)(k+0x02e2) = giuregs[i+1];

   /*=====================================================
    * initial peripheral 
    *=====================================================*/
   /* initial sed1376 (lcd),drivers/video/sed1376-160x240x4.c */   
   pm_sed1376_init();
   pm_lcd_on();
   
   #ifdef CONFIG_BAR_CODE
   /* initial barcode ,driver/char/bar_code.c */
   pm_barcode_init();
   #endif   

#ifdef CONFIG_ZEEVO_BLT
   pm_bluetooth_init();
#endif
   /* initial keyboard */
   pm_kbd_init(kiuregs);
   
   /* clear pwsw statue */
   *(unsigned short *)(0x0a0002b0 + KSEG1) &=~0x0003;
   *(unsigned short *)(0x0a0002ac + KSEG1) &=~0x0007;

   /* initial timer */
#if (CONFIG_PHL4000 || CONFIG_PHL4100)
   pm_timer_init();
#else
   time_init();   
#endif
   sti();
   
   kfree(save_regs_arr);
/* OSE,KEES Wait till everything is initialized 
   pm_timer(PM_RESUME);
   pm_serial(PM_RESUME);
   pm_send_all(PM_RESUME, (void *)0);
*/
   
/* initial timer */
   pm_timer_init();
   
   cli();
   /* initial pwsw */
   *(unsigned short *)(0x0a0002b2 + KSEG1) &= ~0x0003;
   *(unsigned short *)(0x0a0002b0 + KSEG1) &= ~0x0003;
   *(unsigned short *)(0x0a0002b2 + KSEG1) |=  0x0003;
   udelay(1000);
   
   *(unsigned short *)(0x0a0002ae + KSEG1) &= ~0x0007;   
   *(unsigned short *)(0x0a0002ac + KSEG1) &= ~0x0007;
   *(unsigned short *)(0x0a0002ae + KSEG1) |=  0x0007;
   udelay(1000);

   pm_access_device();

   /* turn off LED */
   *(unsigned short *)0xab000100 |=  0x3000;
   *(unsigned short *)0xab000104 &= ~0x3000;
  
   /* turn off buzzer */
   set_freq(0);	/* 2003-0530 */
   sti();   

/* Wait till everything is initialized OSE,KEES */
   pm_timer(PM_RESUME);
   pm_serial(PM_RESUME);
   pm_send_all(PM_RESUME, (void *)0);
   
   return 0;
}

