#ifndef PGU_H
#define PGU_H
#include<linux/types.h>

#define OFFSET 0xa0000000
#define TMOD  0x0a000080 /* Timer Function Set                 */
#define TINT  0x0a000084 /* Timer Interrupt Register           */
#define TINTM 0x0a000086 /* Timer Interrupt Mask Register      */
#define TCNT0 0x0a000088 /* Timer Counter Channel_0 Contol(Up counter) */
#define TLD0  0x0a00008a /* Timer Channel_0 Interval Set       */
#define TGRA0 0x0a00008c /* Timer Channel_0 General-purpose Register A */
#define TGRB0 0x0a00008e /* Timer Channel_0 General-purpose Register B */
#define TGRC0 0x0a000090 /* Timer Channel_0 General-purpose Register C */
#define TCON0 0x0a000092 /* Timer Channel_0 Control */
#define TSEL0 0x0a000094 /* Selector 0 Set */
#define TCAL0 0x0a000096 /* Timer channel_0 Output Operation */ 
#define TCNT1 0x0a000098 /* Timer Counter Channel_1 Control(Up counter) */
#define TLD1  0x0a00009a /* Timer Channel_1 Interval Set     */
#define TGRA1 0x0a00009c /* Timer Channel_1 General-purpose Register A */
#define TGRB1 0x0a00009e /* Timer Channel_1 General-purpose Register B */
#define TGRC1 0x0a0000a0 /* Timer Channel_1 General-purpose Register C */
#define TCON1 0x0a0000a2 /* Timer Channel_1 Contol  */
#define TSEL1 0x0a0000a4 /* Selector 1 Set */
#define TCAL1 0x0a0000a6 /* Timer channel_1 Output Operation */

#define WRITE_TMODE(value)   *( (volatile u16*) (TMOD + OFFSET)  ) = value 
#define WRITE_TINTM(value)   *( (volatile u16*) (TINTM + OFFSET) ) = value
#define WRITE_TCON0(value)   *( (volatile u16*) (TCON0 + OFFSET) ) = value 
#define WRITE_TCNT0(value)   *( (volatile u16*) (TCNT0 + OFFSET) ) = value
#define WRITE_TLD0(value)    *( (volatile u16*) (TLD0 + OFFSET)  ) = value
#define WRITE_TSEL0(value)   *( (volatile u16*) (TSEL0 + OFFSET) ) = value
#define WRITE_TCAL0(value)   *( (volatile u16*) (TCAL0 + OFFSET) ) = value
#define WRITE_TGRA0(value)   *( (volatile u16*) (TGRA0 + OFFSET) ) = value
#define WRITE_TGRB0(value)   *( (volatile u16*) (TGRB0 + OFFSET) ) = value
#define WRITE_TGRC0(value)   *( (volatile u16*) (TGRC0 + OFFSET) ) = value
#define WRITE_TSEL0(value)   *( (volatile u16*) (TSEL0 + OFFSET) ) = value 

/* TCON0 TCON1
 *        Bit name
 * 15:13  TCLK[2:0]   CLK Division Set
 *         TCLK2 TCLK1 TCLK0   Mode
 *           0     0     0  --->CLK
 *           0     0     1  --->CLK/2
 *           0     1     0  --->CLK/4
 *           0     1     1  --->CLK/8
 *           1     0     0  --->CLK/16
 *           1     0     1  --->CLK/32
 *           1     1     0  --->CLK/64
 *           1     1     1  --->CLK/128
 *   12   RFU    Resvred.
 * 11:10  TGRAO[1:0]  GRA Output Compare Set
 *         TGRAO1 TGRAO0       Mode
 *           0     0          Counter Through Output
 *           0     1          Output Compare One-shot Output using GRA
 *           1     0          Output Compare Toggle Output using GRA 
 *           1     1          RFU
 *  9:8   TGRAI[1:0]  GRA Input Capture Set
 *         TGRAI1 TGRAI0
 *           0     0          Input Capture Not Use( TOUTn0 is output pin )
 *           0     1          Input First-transition Capture using GRA
 *           1     0          Input Last-transition Capture using GRA 
 *           1     1          Input First/Last-transition Capture using GRA
 *  7:6   TGRBO[1:0]  GRB Output Compare Set 
 *         TGRBO1 TGRBO0       Mode
 *           0     0          Counter Through Output
 *           0     1          Output Compare One-shot Output using GRB
 *           1     0          Output Compare Toggle Output using GRB
 *           1     1          RFU         
 *  5:4   TGRBI[1:0]  GRB Input Capture Set
 *         TGRBI1 TGRBI0
 *           0     0          Input Capture Not Use( TOUTn1 is output pin )
 *           0     1          Input Rising-edge Capture using GRB
 *           1     0          Input falling-edge Capture using GRB 
 *           1     1          Input Rising/Faling-edge Capture using GRB
 *  3:2   TGRCO[1:0]  GRC Output Compare Set 
 *         TGRCO1 TGRCO0       Mode
 *           0     0          Counter Through Output
 *           0     1          Output Compare One-shot Output using GRC
 *           1     0          Output Compare Toggle Output using GRC
 *           1     1          RFU   
 *  1:0    RFU  Reserved
*/

/* TMOD Configuration(0x0A0000080h)
 *        bit name 
 * 13:15  TDMA[2:0]
 *        TDMA2  TDMA1  TDMA0
 *          0      0      X     DMA not in use
 *          0      1      0     Timer Channle_0 GRA  DMA Effective
 *          0      1      1     Timer Channel_0 GRB  DMA Effectice
 *          1      0      0     Timer Channel_1 GRA  DMA Effective
 *          1      0      1     Timer Channel_1 GRB  DMA Effective
 *          1      1      0     RFU
 *          1      1      1     RFU
 *         *when DMA is effective,input data to register is DMA-transferred
 * 12:10  RFU  Reserved Set 0.
 *   9    TCLR1  Sets counter clear when timer channel_1 is used for input capture
 *               '1' is effective ans '0' is invalid.      
 *   8    TCLR0  Sets counter clear when timer channel_0 is used for input capture
 *               '1' is effective ans '0' is invalid.  
 *  7:6   TSYNCSEL[1:0]  ( Sets synchronizing timer channel in synchronizing operation )
 *        TSYNCSEL[1] TSYNCSEL[0]  
 *             0         0      Timer Channel_0 Synchronize
 *             0         1      Timer Channel_1 Synchronize
 *             1         0      RFU
 *             1         1      Setting Disable
 *   5    RFU  Reserved Set 0.
 *   4    TSYNC1 Sets synchronizing operation of time channel_1
 *               '1' is effective ans '0' is invalid.
 *   3    TSYNC0 Sets synchronizing operation of time channel_0
 *               '1' is effective ans '0' is invalid.       
 *   2    RFU  Reserved Set 0.
 *   1    TENB1 Enables operation of time channel 
 *               '1' is effective ans '0' is invalid. 
 *   0    TENB0 Enables operation of time channel 
 *               '1' is effective ans '0' is invalid. 
 */ 
 
/* TINT Configuration(0x0A000084h)
 *        bit name 
 * 15:10  RFU         Reserved. 
 *                    This bit shall be treated as "Don't Care".                     
 *   9    TGRAIINT1   This bit is set to '1' when GRA input capture interrupt\
 *                    was generated in timer channel_1.
 *                    Write '1' to this bit to clean the interrupt.
 *   8    TGRBIINT1   This bit is set to '1' when GRB input capture interrupt\
 *                    was generated in timer channel_1.
 *                    Write '1' to this bit to clean the interrupt.                                                  
 *   7    TGRAOINT1   This bit is set to '1' when GRA output compare interrupt\
 *                    was generated in timer channel_1.
 *                    Write '1' to this bit to clean the interrupt.
 *   6    TGRBOINT1   This bit is set to '1' when GRB output compare interrupt\
 *                    was generated in time channel_1.
 *                    Write '1' to this bit to clean the interrupt. 
 *   5    TGRCOINT1   This bit is set to '1' when GRC output compare interrupt\
 *                    was generated in time channel_1.
 *                    Write '1' to this bit to clean the interrupt. 
 *   4    TGRAIINT0   This bit is set to '1' when GRA input capture interrupt\
 *                    was generated in timer channel_0.
 *                    Write '1' to this bit to clean the interrupt.
 *   3    TGRBIINT0   This bit is set to '1' when GRB input capture interrupt\
 *                    was generated in timer channel_0.
 *                    Write '1' to this bit to clean the interrupt.                                                  
 *   2    TGRAOINT0   This bit is set to '1' when GRA output compare interrupt\
 *                    was generated in timer channel_0.
 *                    Write '1' to this bit to clean the interrupt.
 *   1    TGRBOINT0   This bit is set to '1' when GRB output compare interrupt\
 *                    was generated in time channel_0.
 *                    Write '1' to this bit to clean the interrupt. 
 *   0    TGRCOINT0   This bit is set to '1' when GRC output compare interrupt\
 *                    was generated in time channel_0.
 *                    Write '1' to this bit to clean the interrupt. 
 */  

/* TINTM Configuration(0x0A000086h)
 *        bit name 
 * 15:10  RFU         Reserved. Set 0.                     
 *   9    TGRAIMSK1   Timer Channel_1 GRAIINT Mask.
 *   8    TGRBIMSK1   Timer Channel_1 GRBIINT Mask.                                                 
 *   7    TGRAOMSK1   Timer Channel_1 GRAOINT Mask.
 *   6    TGRBOMSK1   Timer Channel_1 GRBOINT Mask.
 *   5    TGRCOMSK1   Timer Channel_1 GRCOINT Mask.
 *   4    TGRAIMSK0   Timer Channel_0 GRAIINT Mask.
 *   3    TGRBIMSK0   Timer Channel_0 GRBIINT Mask.                                                  
 *   2    TGRAOMSK0   Timer Channel_0 GRAOINT Mask.
 *   1    TGRBOMSK0   Timer Channel_0 GRBOINT Mask.
 *   0    TGRCOMSK0   Timer Channel_0 GRCOINT Mask.
 *                    *(Enables interrupe '1' 
 *                      Disable interrupt '0' )* 
 */  
 
/* TLD0  TLD1
 * TLD 15:0
 * when the value of timer counter become equal with this register,
 * counting operation starts form initial value(all "0" ),
 * this bit is used to set timer interval  
*/
 
 
 
#endif /* PGU_H */   
