/*
 * linux/arch/mips/vr41xx/kbd.c
 *
 * VR41xx keyboard (KIU) driver
 *
 * Copyright (C) 1999 Bradley D. LaRonde
 * Oct 1999 Hiroshi Kawashima (kawashima@iname.com) adjusted scan lines for MobilPro.
 * Copyright (C) 2000 Michael Klar <wyldfier@iname.com>
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * 2001/11/1 ADD KIU control by sam <sam_liao@coventive.com>
 *
 */

#include <linux/sched.h>
#include <linux/tty.h>
#include <linux/mm.h>
#include <linux/kbd_kern.h>
#include <linux/errno.h>
#include <linux/init.h>
#include <linux/pm.h>
#include <asm/keyboard.h>
#include <linux/kbd_ll.h>
#include <asm/vr41xx.h>
#include <asm/vr41xx-platdep.h>
#include <asm/uaccess.h>
// I don't know about other devices, but WorkPad doesn't have LEDs for
// the lock states, rather those are handled on screen.  Maybe we should
// send an event notification to user space when LED state changes?  MFK

#define KEY_CONTROL_MAJOR 127
#define KEY_GET_MODE 0
#define KEY_SET_MODE 1
#define KEY_GET_KEYCODE 2 
#define KEY_PUSH_KEY 3

#if (CONFIG_ROAD_HT || CONFIG_PHL4000 || CONFIG_AXIS_H3ABB || CONFIG_4100)
#define ROAD_HARDWARE_KEY
#endif

#ifdef ROAD_HARDWARE_KEY
	#define KEY_DISABLE 0
	#define ENGLISH_SMALL_MODE 1 
	#define NUMERIC_MODE 2
	#define ENGLISH_CAPITAL_MODE 3
	extern void set_road_keymode(int mod);
	extern int get_road_keymode(void);
	typedef struct KEYCODE{
	        unsigned char data[6];
	}keycode;
static keycode keydata;
#endif


extern void PushKey(int key);

static int key_open(struct inode *inode, struct file *filp);
static int key_close(struct inode *inode, struct file *filp);
static int key_ioctl(struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg);

//------------------------------------------------------------
static struct file_operations key_control_fops = {
	ioctl:		key_ioctl,
	open:		key_open,
	release:	key_close,
};

//-------------------------------------------------------------

int hkey_enable(int enable)
{
 unsigned short reg;
 reg = *VR41XX_KIUSCANREP;
 	if(enable) {
  	*VR41XX_KIUSCANREP = reg | VR41XX_KIUSCANREP_KEYEN ;
 	}
 	else{
  	*VR41XX_KIUSCANREP = reg & ~VR41XX_KIUSCANREP_KEYEN;
 	}
 return 0;
}

static void load_keydata(void)
{
        keydata.data[0] = (unsigned char)(*VR41XX_KIUDAT0 & 0x00ff) ;
        keydata.data[1] = (unsigned char)((*VR41XX_KIUDAT0 >> 8) & 0x00ff ) ;
        keydata.data[2] = (unsigned char)(*VR41XX_KIUDAT1 & 0x00ff) ;
        keydata.data[3] = (unsigned char)((*VR41XX_KIUDAT1 >> 8) & 0x00ff ) ;
        keydata.data[4] = (unsigned char)(*VR41XX_KIUDAT2 & 0x00ff) ;
#ifdef CONFIG_PHL4000	
        *VR41XX_GIUIOSELH &= ~0x0010;//GPIO20 input
        keydata.data[5] = (unsigned char)( ~(*VR41XX_GIUPIODH >> 4) & 0x0001 );
#endif
#ifdef CONFIG_PHL4100	
        keydata.data[5] = (unsigned char)((*VR41XX_KIUDAT2 >> 8) & 0x00ff ) ;
#endif	
               
}

//---------------------------------------------------------------------
static int key_open(struct inode *inode, struct file *filp)
{
    return 0;
}
static int key_close(struct inode *inode, struct file *filp)
{
    return 0;
}

static int key_ioctl(struct inode *inode, struct file *file,
			unsigned int cmd, unsigned long arg)
{
 int ret = (-EINVAL);
 int key_mode_set;
 
 switch(cmd){
      case KEY_PUSH_KEY:
           if (copy_from_user(&ret, (void *)arg, sizeof(ret)))
               return -EFAULT;
           PushKey(ret);
           return 0;
      case KEY_GET_KEYCODE:
#ifdef ROAD_HARDWARE_KEY
           load_keydata();
           return copy_to_user( (void *)arg,&keydata,sizeof(keydata) )?-EFAULT:0 ;
#endif
           break;
	   
      case KEY_GET_MODE:
#ifdef ROAD_HARDWARE_KEY      
      key_mode_set = get_road_keymode();
#endif      
      return copy_to_user( (void *)arg,&key_mode_set,sizeof(int) )?-EFAULT:0 ;

      case KEY_SET_MODE:
      if (copy_from_user(&key_mode_set, (void *)arg, sizeof(key_mode_set)))
      return -EFAULT;
#ifdef ROAD_HARDWARE_KEY
       if(key_mode_set == 0 ) hkey_enable(0);
       else
       if(key_mode_set == 1 ) set_road_keymode(ENGLISH_SMALL_MODE);
       else
       if(key_mode_set == 2 ) set_road_keymode(NUMERIC_MODE);
       else
       if(key_mode_set == 3 ) set_road_keymode(ENGLISH_CAPITAL_MODE);
       else
       if(key_mode_set == 4 ) hkey_enable(1);
#endif       
       return 0;

   }
      return ret;
}
//----------------------------------------------------------

int __init key_control_init(void)
{
#ifdef ROAD_HARDWARE_KEY
  int result;
  int key_control_major = KEY_CONTROL_MAJOR;

  result = register_chrdev(key_control_major,"key_control",
						 &key_control_fops);
  if (result < 0) {
    printk(KERN_WARNING " KEY_CONTROL_MAIOR: cant get major %d\n",
						 key_control_major);
    return result;
  }
  set_road_keymode(NUMERIC_MODE);
  printk("key_control Initialized...\n");
#endif  
  return 0;

}
//----------------------------------------------------------







void kbd_leds(unsigned char leds)
{
	return;
}

// scancode -> keycode translation not supported, since all scancodes
// can be addressed in the keymap

int kbd_setkeycode(unsigned int scancode, unsigned int keycode)
{
	return (scancode == keycode) ? 0 : -EINVAL;
}

int kbd_getkeycode(unsigned int scancode)
{
	return scancode;
}

int kbd_translate(unsigned char scancode, unsigned char *keycode,
	char raw_mode)
{
	*keycode = scancode;
	return 1;
}

char kbd_unexpected_up(unsigned char keycode)
{
//	printk(KERN_WARNING "vr41xx_kiu: unexpected up, keycode 0x%x\n", keycode);
	return 0x80;
}

#define DEFAULT_REP_WAIT (HZ/4)
#define DEFAULT_REP_RATE (HZ/25)

static int rep_scancode = -1;
static unsigned long rep_wait = DEFAULT_REP_WAIT;
static unsigned long rep_rate = DEFAULT_REP_RATE;
static unsigned long next_reptime;

inline static void handle_cmp_kiudat_bit(unsigned short kiudat, unsigned short cmp_kiudat,
	int scancode, unsigned short bitmask)
{
	int down;

	// did this bit change?
	if ( cmp_kiudat & bitmask ) {
#ifdef CONFIG_CPU_VR4181
		down = kiudat & bitmask ? 0 : 1;
#else
		down = kiudat & bitmask ? 1 : 0;
#endif
		if (down) {
			// on press, set up repeat
			rep_scancode = scancode;
			next_reptime = jiffies + rep_wait;
		} else {
			// on release, stop repeating
			if (rep_scancode == scancode)
				rep_scancode = -1;
		}

		// send the scancode and down or up to higher level code
		handle_scancode(scancode, down);
	}
}

inline static void handle_cmp_kiudat_nbl(unsigned short kiudat, unsigned short cmp_kiudat,
	int scancode, unsigned short nblmask)
{
	unsigned short bitmask;

	// common case if for nothing set
	if (cmp_kiudat & nblmask) {
		// handle each bit
		for (bitmask = nblmask & 0x1111 ; bitmask & nblmask; bitmask <<= 1)
			handle_cmp_kiudat_bit(kiudat, cmp_kiudat, scancode++, bitmask);
	}
}

inline static void handle_kiudat(unsigned short kiudat, unsigned short last_kiudat,
	int scancode_base)
{
	unsigned short cmp_kiudat, nblmask;
	int scancode = scancode_base;

	// compare this data with the previous data, setting only bits that changed
	cmp_kiudat = kiudat ^ last_kiudat;

	// nothing changed - most common case
	if ( cmp_kiudat == 0 )
		return;

	for (nblmask = 0x000f; nblmask; nblmask <<= 4) {
		// handle each nibble
		handle_cmp_kiudat_nbl(kiudat, cmp_kiudat, scancode, nblmask);
		scancode += 4;
	}
}

inline static void handle_kiu_data_ready(void)
{
#ifdef CONFIG_CPU_VR4181
	static unsigned short last_kiudat[8] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff};
#else
	static unsigned short last_kiudat[6];
#endif
	unsigned short thisdat, lastdat;
	volatile unsigned short *kiudatreg = VR41XX_KIUDAT0;
	int i;

	// For each KIU register.
	for (i = 0; i < (sizeof(last_kiudat) / sizeof(last_kiudat[0])); i++) {
		lastdat = last_kiudat[i];
		thisdat = last_kiudat[i] = *(kiudatreg++);
		handle_kiudat(thisdat, lastdat, i * 16);
	}
#if 0 //CV -> MASK repeat :SAM
	if ((rep_scancode >= 0) && (time_after_eq(jiffies, next_reptime))) {
		next_reptime = jiffies + rep_rate;
		handle_scancode(rep_scancode, 1);
	}
#endif	
}

void kbd_irq_handler(int irq, void *dev_id, struct pt_regs *regs)
{
	unsigned short kiuint;

	// get keyboard interrupt status
	kiuint = *VR41XX_KIUINT;
	barrier();

	// ack them, but only the ones that were read.	if we just ack
	// all of them, we run the risk of ignoring interrupts that
	// happen between these 2 lines
	*VR41XX_KIUINT = kiuint;

	// keyboard data ready?
	if ( kiuint & VR41XX_KIUINT_KDATRDY ) {
#ifdef DEBUG_KIU
		printk("kiudat %x %x %x %x %x %x %x %x\n",
			*VR41XX_KIUDAT0, *VR41XX_KIUDAT1, *VR41XX_KIUDAT2, *VR41XX_KIUDAT3,
			*VR41XX_KIUDAT4, *VR41XX_KIUDAT5, *VR41XX_KIUDAT6, *VR41XX_KIUDAT7);
#endif
		handle_kiu_data_ready();
	}

	// lost keyboard data?
	if ( kiuint & VR41XX_KIUINT_KDATLOST ) {
		printk(KERN_DEBUG "vr41xx_kiu: keyboard data lost\n");
	}
}

#ifdef CONFIG_PM
static int pm_kbd_request(struct pm_dev *dev, pm_request_t rqst, void *data)
{
	static unsigned short save_regs[4];

	switch (rqst) {
	case PM_SUSPEND:
		// Save hardware registers.
#ifndef CONFIG_CPU_VR4181
		save_regs[0] = *VR41XX_SCANLINE;
#endif
		save_regs[1] = *VR41XX_KIUSCANREP;
		save_regs[2] = *VR41XX_KIUWKS;
		save_regs[3] = *VR41XX_KIUWKI;

// MFK: there should be some kind of enable/disable for this (ie, "enable
// wake-up on this device"), but user interface for that not worked out yet.
		*VR41XX_MKIUINTREG = VR41XX_KIUINT_SCANINT;
		break;
	case PM_RESUME:
		// Restore hardware registers.
		// NOTE:  Maybe this only needs to be done after certain power states.	bdl
#ifndef CONFIG_CPU_VR4181
		*VR41XX_SCANLINE =   save_regs[0];
#endif
		*VR41XX_KIUSCANREP = save_regs[1];
		*VR41XX_KIUWKS =     save_regs[2];
		*VR41XX_KIUWKI =     save_regs[3];

		*VR41XX_MKIUINTREG = VR41XX_KIUINT_KDATRDY | VR41XX_KIUINT_KDATLOST;
		break;
	}
	return 0;
}
#endif

void __init kbd_init_hw(void)
{
	printk(KERN_INFO "Keyboard driver version 0.2a for VR41XX KIU\n");

#ifndef CONFIG_CPU_VR4181
	// turn on the KIU clock
	*VR41XX_CMUCLKMSK |= 0x0008;

	// reset the KIU
	*VR41XX_KIURST = 0x0001;

	barrier();

	// number of scanlines to use varies by device, but most use all 12
#if (KBD_SCANLINES == 8)
	*VR41XX_SCANLINE = 0x0002;
#elif (KBD_SCANLINES == 10)
	*VR41XX_SCANLINE = 0x0001;
#elif (KBD_SCANLINES == 12)
	*VR41XX_SCANLINE = 0x0000;
#else
#error "Illegal number of scanlines defined in KBD_SCANLINES"
#endif
#endif

	// enable scan, auto scan, auto stop, and one stop rep
	*VR41XX_KIUSCANREP = VR41XX_KIUSCANREP_KEYEN |
		 VR41XX_KIUSCANREP_ATSCAN | VR41XX_KIUSCANREP_ATSTP | (1 << 4);

	// set stabilization, off, and wait times in 30us increments
	// this uses a short (but not the shortest) setting for each
#ifdef CONFIG_CPU_VR4181
	// VR4181 off time is fixed at 1
	*VR41XX_KIUWKS = (9-1) | ((17-1) << 10);
#else
//	*VR41XX_KIUWKS = (9-1) | ((9-1) << 5) | ((9-1) << 10);
	*VR41XX_KIUWKS = (12-1) | ((12-1) << 5) | ((12-1) << 10);
#endif

	// set wait time between scan cycles in 30us increments
	// setting this too short can result in lost keyboard data
	// this wait is on top of the stabilization, off, and wait times above,
	// which are per scan line, so total cycle time is about 1/50 sec
//	*VR41XX_KIUWKI = (20000 / 30) - KBD_SCANLINES * 27;
	*VR41XX_KIUWKI = 0x03ff;

	barrier();

	// clear any pending interrupts
	*VR41XX_KIUINT = 0x0007;

	barrier();

	// install the interrupt handler
	request_irq(VR41XX_IRQ_KIU, kbd_irq_handler, 0, "keyboard", NULL);

	// unmask the keyboard interrupts that we are interested in
	*VR41XX_MKIUINTREG = VR41XX_KIUINT_KDATRDY | VR41XX_KIUINT_KDATLOST;

#ifdef DEBUG_KIU
	printk("kiudat %x %x %x %x %x %x %x %x\n",
		*VR41XX_KIUDAT0, *VR41XX_KIUDAT1, *VR41XX_KIUDAT2, *VR41XX_KIUDAT3,
		*VR41XX_KIUDAT4, *VR41XX_KIUDAT5, *VR41XX_KIUDAT6, *VR41XX_KIUDAT7);
#endif

#ifdef CONFIG_PM
	pm_register(PM_SYS_DEV, PM_SYS_KBC, pm_kbd_request);
#endif
}
