/*************************************************************
 *  ROAD led driver
 *
 *  linux/drivers/led/leddev.c 
 *
 * Initial Version by: sam liao (sam_liao@coventive.com)
 *
 * This driver provides  control for the led on_off
 * red and green.
 *
 * History:
 *      9/24/2001 - sam liao <sam_liao@coventive.com)
 *                  Initial Version. 
 *      8/02/2002 - Add Red/Green LED off command
 *      	RED_OFF and GREEN_OFF ioctl	
 * 
 *************************************************************/

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/init.h>
#include <asm/vr4121/vr4121.h>
#include <asm/leddev.h>

#define LED_VERSION	"2003-03-10 V1.0"

//------------------------------------------------------------
static int led_control_open(struct inode *inode, struct file *filp);
static int led_control_close(struct inode *inode, struct file *filp);
static int led_control_ioctl(struct inode * inode, struct file *filp,
                                   unsigned int cmd , unsigned long arg);

static void led_red_on(void);
static void led_green_on(void);
static void red_green_on(void);
static void led_off(void);
static void red_off(void);
static void green_off(void);
int __init led_control_init(void);

//------------------------------------------------------------
static struct file_operations led_control_fops;
//------------------------------------------------------------
static void led_red_on(void)
{
 /*set GPIO12 is Lo GPIO13 is Hi */
 *( ( volatile unsigned short *)VR41XX_GIUPIODL ) |= LED_RED_PIN;
// *( ( volatile unsigned short *)VR41XX_GIUPIODL ) &= ~LED_GREEN_PIN;
}

static void led_green_on(void)
{
 /*set GPIO12 is Hi and GPIO13 is Lo */
// *( ( volatile unsigned short *)VR41XX_GIUPIODL ) &= ~LED_RED_PIN;
 *( ( volatile unsigned short *)VR41XX_GIUPIODL ) |= LED_GREEN_PIN;
}


static void led_off(void)
{
 /*set GPIO12 and GPIO13 to Lo*/
 *( ( volatile unsigned short *)VR41XX_GIUPIODL ) &= ~LED_RED_PIN;
 *( ( volatile unsigned short *)VR41XX_GIUPIODL ) &= ~LED_GREEN_PIN;
}

static void red_off(void)
{
	*( ( volatile unsigned short *)VR41XX_GIUPIODL ) &= ~LED_RED_PIN;
}

static void green_off(void)
{
	*( ( volatile unsigned short *)VR41XX_GIUPIODL ) &= ~LED_GREEN_PIN;
}

void static vr4121_led_pin_init(void)
{
  /*set GPIO12 and GPIO13 is output*/
  *( ( volatile unsigned short *)VR41XX_GIUIOSELL ) |= LED_RED_PIN;
  *( ( volatile unsigned short *)VR41XX_GIUIOSELL ) |= LED_GREEN_PIN;
}

static void delay_time(int time)
{
 unsigned long Timeout;
 Timeout = jiffies + time;
    while(1)
    {
     if(jiffies >= Timeout ) break;
    }
}

void low_power_led(void)
{
  led_red_on();
  delay_time(250);
  led_off();
}


//---------------------------------------------------------------------
static int led_control_open(struct inode *inode, struct file *filp)
{
    MOD_INC_USE_COUNT;
    return 0;
}
static int led_control_close(struct inode *inode, struct file *filp)
{
	MOD_DEC_USE_COUNT;
    return 0;
}

static int led_control_ioctl(struct inode * inode, struct file *filp,
				   unsigned int cmd , unsigned long arg)
{
	int ret = 0;
	
	switch(cmd) 
	  {
          case RED_ON:
		led_red_on();
	    	break;
	  case GREEN_ON:
		led_green_on();
	    	break;
	  case LED_OFF:
		led_off();
	    	break;
	  case LOW_POWER:
	        low_power_led();	 
	  case RED_OFF:
	        red_off();
	        break;
	  case GREEN_OFF:
	        green_off();
	        break;
	  default:
		ret = (-EINVAL); 
	    	break;
	  }
	
	return ret;
}

//------------------------------------------------------------
int __init led_control_init(void)
{
  int result;
  int led_control_major;
  led_control_major = LED_CONTROL_MAJOR;

  result = register_chrdev(led_control_major,LED_CONTROL_NAME,
						 &led_control_fops);
  if (result < 0) {
    printk(KERN_WARNING " LED CONTROL MAIOR: cant get major %d\n", 
	  					 led_control_major);
    return result;
  }
  vr4121_led_pin_init();
  printk(KERN_INFO "LED Driver v" LED_VERSION "\n");
  //printk("led_control Driver Initialized...\n");
  led_off();
  return 0;

}

//------------------------------------------------------------
static struct file_operations led_control_fops = {
        ioctl:          led_control_ioctl,
        open:           led_control_open,
        release:        led_control_close,
};

//-------------------------------------------------------------

