/*
 * linux/arch/mips/vr41xx/roadht/setup.c
 *
 * Setup routines for the Opto H-3A
 *
 * Copyright (C) 2002 Jalen Yang <jalen@coventive.com>
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 */

#include <linux/init.h>
#include <asm/vr41xx.h>
#include <asm/mr2a.h>

#include <linux/config.h>
#include <linux/console.h>
#include <linux/ide.h>
#include <linux/ioport.h>
 
#include <asm/reboot.h>
#include <asm/time.h>
#include <asm/vr41xx/roadht.h>

#define    PCMCIA_3E0     0xb40003E0
#define    PCMCIA_3E1     0xb40003E1

#ifdef CONFIG_BLK_DEV_INITRD
extern unsigned long initrd_start, initrd_end;
extern void * __rd_start, * __rd_end;
#endif
 
#ifdef CONFIG_BLK_DEV_IDE
extern struct ide_ops std_ide_ops;
#endif

#ifdef CONFIG_ROAD_HT
void __init opto_h3a_setup (void);
#endif

void __init nec_vr41xx_setup (void)
{
	
#ifdef CONFIG_ROAD_HT
opto_h3a_setup();
#endif
  set_io_port_base (IO_PORT_BASE);
  ioport_resource.start = IO_PORT_RESOURCE_START;
  ioport_resource.end = IO_PORT_RESOURCE_END;
  iomem_resource.start = IO_MEM_RESOURCE_START;
  iomem_resource.end = IO_MEM_RESOURCE_END;

#ifdef CONFIG_BLK_DEV_INITRD
  ROOT_DEV = MKDEV (RAMDISK_MAJOR, 0);
  initrd_start = (unsigned long) &__rd_start;
  initrd_end = (unsigned long) &__rd_end;
#endif

  _machine_restart = vr41xx_restart;
  _machine_halt = vr41xx_halt;
  _machine_power_off = vr41xx_power_off;

  board_time_init = vr41xx_time_init;
  board_timer_setup = vr41xx_timer_setup;

#ifdef CONFIG_FB
  conswitchp = &dummy_con;
#endif

#ifdef CONFIG_BLK_DEV_IDE
  ide_ops = &std_ide_ops; 
#endif

  vr41xx_bcu_init ();

  vr41xx_cmu_init (0);
 
#ifdef CONFIG_IRDA
  
 	#if (CONFIG_PHL4000 || CONFIG_PHL4100)  
	vr41xx_siu_init (SIU_IRDA, IRDA_SHARP);
	//vr41xx_siu_init (SIU_IRDA, IRDA_TEMIC);
	//vr41xx_siu_init (SIU_IRDA, IRDA_HP);
	
	#endif
  
	#ifdef CONFIG_AXIS_H3ABB
	 vr41xx_siu_init (SIU_IRDA, IRDA_HP);
	#endif
  
#else
	vr41xx_siu_init (SIU_RS232C, 0);
#endif  
      
}

void __init opto_h3a_setup (void)
{
  printk ("opto_h3a_setup\n");

  /* Set GPIO26 to be input mode */
  *((volatile unsigned short *)0xab000102) &= ~0x0400;
  
#if CONFIG_NET_ETHERNET
  // CPU_GPIO18 INPUT
  *VR41XX_GIUIOSELH &= 0xfffb;

  // CPU_GPIO18 to Edge Trigger
  *VR41XX_GIUINTTYPH |= 0x0004;

  // CPU_GPIO18 signal Hold in GIU
  *VR41XX_GIUINTHTSELH |= 0x0004;
  // MR2A Programmable Decoder
 
  *MR2A_PDEC1LL = 0x30;
  *MR2A_PDEC1HL = 0x3ff;
#endif

#if CONfIG_BAR_CODE
#if 0
  /* Jeff 5/16 */
  *(unsigned short *) (MGIUINTHREG) |= 0x0010;
  *(unsigned short *) (MSYSINT1REG) |= 0x0100;
#endif
       #define BARCODE_IRQPIN
       	vr41xx_set_irq_trigger(BARCODE_IRQPIN, TRIGGER_EDGE, SIGNAL_HOLD); 
       #ifdef PHL4000
       #define  BARCODE_TRIG_PIN 20
       	vr41xx_set_irq_trigger(BARCODE_TRIG_PIN, TRIGGER_EDGE, SIGNAL_HOLD); 
  	printk ("Enable GPIO20 interrupt\n");
       #endif
	#if 0
	  /* GPIO 20 as interrupt pin */
	  *(unsigned short *) (GIUIOSELH) &= 0xffef;
	  *(unsigned short *) (GIUINTTYPH) |= 0x0010;
	  *(unsigned short *) (GIUINTENH) |= 0x0010;
	  *(unsigned short *) (GIUINTHTSELH) |= 0x0010;
	  *(unsigned short *) (GIUINTSTATH) |= 0x0010;
	#endif  
#endif

#if 0
  #if (CONFIG_PHL4000 || CONFIG_PHL4100)
  
       printk("Set MR2A GPIO1 = Low, PCMCIA CS# = low\n"); /* Jeff 0613 */
       *(unsigned short *)(0xaa00005c) = 0x0000;
       *(unsigned short *)(0xaa000040) |= 0x0002;
       *(unsigned short *)(0xaa000044) &= ~0x0002;
       
  #else
       /* GPIO19 for CS# */
       printk("config cpu-gpio19 lo cs==enable, mr2a-gpio0 input\n");
       *(unsigned short *)(VR41XX_GIUIOSELH)   |=  0x0008 ;     //CPU-GPIO19-OUTPUT
       *(unsigned short *)(VR41XX_GIUPIODH)    &= ~0x0008 ;     //CPU-GPIO19-LOW
       *(unsigned short *)(0xaa000040)  &= ~0x0001 ;     //MR2-GPIO0-INPUT
  #endif

         // CPU_GPIO17 INPUT
	   *VR41XX_GIUIOSELH &= ~0x0002;
         // CPU_GPIO17 to Edge Trigger
	   *VR41XX_GIUINTTYPH |= 0x0002;
	 // CPU_GPIO17 signal Hold in GIU
	   *VR41XX_GIUINTHTSELH |= 0x0002;

	   *(volatile unsigned char *)( PCMCIA_3E0 ) = 0x2f;
	   *(volatile unsigned char *)( PCMCIA_3E1 ) = 0x0b; // 0x09;
	   
#endif

}
