/* vi: set sw=4 ts=4: */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/fs.h>
#include <linux/devfs_fs_kernel.h>
#include <asm/uaccess.h>
#include <linux/init.h>
#include <asm/vr4121/vr4121.h>
#include <linux/mm.h>
#include <linux/delay.h>
#include "MR-2A_PGU.h"
#include <asm/sp.h>
#include <linux/sched.h>  

#define Buzzer_VERSION	"2003-03-10 V1.0"

struct file_operations buzzer_speaker_fops;

static struct timer_list Buzz_Off_Timer;
static struct timer_list Buzz_On_Timer;
static struct timer_list BuzzerHeartbeat;

static void sound_init(void);
static void MR2A_PGU_reg_config_init(void);
static int VR4121_SP_reg_config(void);
static void set_volume(VOLUME volume);
void set_freq(int freq);//freq 20HZ-20KHZ	/* 2003-0530 */
static void Buzzer(void);
static void sound(unsigned int freq,VOLUME vol);
static void buzzer_sound(void);
static void buzzer_off_function(void);
static void buzzer_on_function(void);
static int sp_open(struct inode * inode, struct file * file);
static int sp_close(struct inode *inode, struct file *filp);
static int sp_ioctl(struct inode *inode, struct file *file,	unsigned int cmd, unsigned long arg);

void __init sp_init(void);
void keyboard_irq_buzz(void);
void low_power_buzz(void);
void barcode_reading_buzz(void);
void sound_PHL4000_barcode_on(void);
void sound_PHL4000_barcode_off(void);
void buzz_poweroff_init(void);

static int Number_iten = 0;
static BUZZER system_buzz[4];
static BUZZER want_buzz;
static BUZZER *ptmp_buzz = NULL;
static OSE_BUZZER ose_buzzer;

static void ose_buzzer_function(void);

void low_power_buzz(void)
{  
	int i ;
  	for( i=0 ; i<3 ; i++ )
    {
    	sound(1200,volume_hi);
      	mdelay(200);
      	set_freq(0);
      	mdelay(50);
    } 
}
    
static void sound_init(void)
{
	set_freq(0);
 /* barcoder reading sound ---------0 */
    system_buzz[BARCODE_READ].freq = 500;
    system_buzz[BARCODE_READ].buzz_on_time = 2;
    system_buzz[BARCODE_READ].buzz_off_time = 1;
    system_buzz[BARCODE_READ].times = 1;
    system_buzz[BARCODE_READ].volume = volume_hi;
 /* keyboard input -----------------1 */
    system_buzz[KEYBOARD_IN].freq = 1000;
    system_buzz[KEYBOARD_IN].buzz_on_time = 6;
    system_buzz[KEYBOARD_IN].buzz_off_time = 1;
    system_buzz[KEYBOARD_IN].times = 1;
    system_buzz[KEYBOARD_IN].volume = volume_hi;
 /*  received data form server -----2 */
    system_buzz[DATA_RECEIVER].freq = 2000;
    system_buzz[DATA_RECEIVER].buzz_on_time = 2;
    system_buzz[DATA_RECEIVER].buzz_off_time = 2;
    system_buzz[DATA_RECEIVER].times = 2;
    system_buzz[DATA_RECEIVER].volume = volume_hi;
 /* alarm sound --------------------3 */
    system_buzz[ALARM].freq = 500;
    system_buzz[ALARM].buzz_on_time = 50;
    system_buzz[ALARM].buzz_off_time = 1;
    system_buzz[ALARM].times = 1;
    system_buzz[ALARM].volume = volume_hi;
    //---------------------------------------
 
	init_timer(&Buzz_Off_Timer);
 	init_timer(&Buzz_On_Timer);
	init_timer(&BuzzerHeartbeat);
 	Buzz_Off_Timer.function = ( void (*)(unsigned long) ) buzzer_off_function;
 	Buzz_On_Timer.function = ( void (*)(unsigned long) ) buzzer_on_function;
	BuzzerHeartbeat.function = ( void (*)(unsigned long) ) ose_buzzer_function;
}

static void MR2A_PGU_reg_config_init(void)
{
 	WRITE_TMODE(0x0001); /* use timer channel 0 */
 	WRITE_TINTM(0x0007); /* enables GRAOINT GRBOINT GRCOINT */
 	WRITE_TCON0(0x0888); /* clk/1
						  * GRA GRB GRC comper toggle output
		       			  * GRA GRB GRC input capture not use
		      			 */
	WRITE_TSEL0(0x1000);/* output select form GRA */
 	WRITE_TSEL0(0x0000);
 	WRITE_TCAL0(0x0000);/* through */
}


static int VR4121_SP_reg_config(void)
{
	CONFIG_GIO14_IS_OUTPIN();
	CONFIG_GIO14_NOT_USE_PULL_UP_DOWN();
	return(0);
}

static void set_volume(VOLUME volume)
{
	switch(volume)
	{
   		case volume_hi:
			VOL_H();
   			break;

   		case volume_lo:
			VOL_L();
   			break;

   		case volume_off:
			set_freq(0);
   			break;
	}
}

//static void set_freq(int freq)//freq 20HZ-20KHZ
void set_freq(int freq)//freq 20HZ-20KHZ
{
	u16 GRA_init_value = 0;
  	u16 TLD_init = 0;
  	u16 tld = 0;
	    
	WRITE_TMODE(0x0001);
	WRITE_TINTM(0x0000);
	WRITE_TCON0(0x0800);
	WRITE_TSEL0(0x0000);
	WRITE_TCAL0(0x0000);/* through */
		 
	if(freq == 0)
 	{
    	WRITE_TMODE(0x0000);
    	TLD_init = 0x0000;
    	GRA_init_value = 0xffff;
    	WRITE_TLD0(TLD_init);
    	WRITE_TCNT0(0x0000);
    	WRITE_TGRA0(GRA_init_value);
		WRITE_TSEL0(0x0000);//and x,y
  		WRITE_TCAL0(0x5000);
    	return ;
  	}
  
  	if( (freq < 800) )
  	{
   		WRITE_TCON0(0xE800);
   		freq *= 128;
  	}
	 
  	tld = 48000000/freq;
  	TLD_init = (u16)(tld/4);
  	GRA_init_value = TLD_init >> 1 ;
  	WRITE_TLD0(TLD_init);
  	WRITE_TCNT0(0x0000);  /* Up counter */
  	WRITE_TGRA0(GRA_init_value); /* Duty cycle = 50% */
}

static void Buzzer(void)
{
  	VR4121_SP_reg_config();
  	MR2A_PGU_reg_config_init();
  	set_freq(want_buzz.freq);
  	set_volume(want_buzz.volume);
}

void buzz_poweroff_init(void)
{
	set_freq(0);
}

static void sound(unsigned int freq,VOLUME vol)
{
	VR4121_SP_reg_config();
  	MR2A_PGU_reg_config_init();
  	set_freq(freq);
  	set_volume(vol);
}

void keyboard_irq_buzz(void)
{
	extern int buzzflag;
	if ( buzzflag == 1 ) 
		return;

	if (system_buzz[KEYBOARD_IN].freq == 0)
		return;
		
	memcpy(&want_buzz,&system_buzz[KEYBOARD_IN],sizeof(BUZZER));
 	buzzer_sound();
}

void __init sp_init(void)
{
	printk(KERN_INFO "Buzzer Driver v" Buzzer_VERSION "\n");

	if( devfs_register_chrdev(SP_MAJOR, SP_NAME, &buzzer_speaker_fops) == -1 )
	{
		printk(KERN_ERR "sp: initial error! sp device already in use.\n");
	    return;
	}
	sound_init();
	return ;
}

static void buzz_abort(void)
{
	del_timer(&Buzz_On_Timer);
	del_timer(&Buzz_Off_Timer);
	del_timer(&BuzzerHeartbeat);
	set_freq(0);
}

static void buzzer_sound(void)
{
	Buzzer();
	mod_timer(&Buzz_Off_Timer,jiffies + want_buzz.buzz_on_time);	
}
	
static void buzzer_off_function(void)
{
	set_freq(0);
  	if( --want_buzz.times )
	{
	  	mod_timer(&Buzz_On_Timer,jiffies + want_buzz.buzz_off_time);	
	}
}

static void buzzer_on_function(void)
{
	Buzzer();
    mod_timer(&Buzz_Off_Timer,jiffies + want_buzz.buzz_on_time);	
}
	
static int sp_open(struct inode * inode, struct file * file)
{
	MOD_INC_USE_COUNT;
	return 0;
}

static int sp_close(struct inode *inode, struct file *filp)
{
	MOD_DEC_USE_COUNT;
	return 0;
}

void sound_PHL4000_barcode_on(void)
{
	sound(3000,volume_hi);
}


void sound_PHL4000_barcode_off(void)
{
    set_volume(volume_off);
}

static void ose_buzzer_function(void)
{
	int freq;
	int idx;
	
	idx = ose_buzzer.note++;
	freq = ose_buzzer.freq[idx];
	
	if ((freq == 0) || (idx >= 20))
	{
		set_freq(0);
		del_timer(&BuzzerHeartbeat);
		return;
	}

	set_freq(0);
	if (freq != 1)			// SPAUSE
		sound(freq,ose_buzzer.volume);

	mod_timer(&BuzzerHeartbeat,jiffies + 2*ose_buzzer.duration);
}


static int sp_ioctl(struct inode *inode, struct file *file,unsigned int cmd, unsigned long arg)
{
	BUZZER *buzz;
	int frequency;
	int volume;
	int number;
	int buzz_on_time;
	int times;
printk("buzzer ioctl\n");	
	switch( cmd )
	{
		case SP_IOC_FREQ:
			if (copy_from_user(&frequency, (void *)arg, sizeof(int)))
		   		return -EFAULT;
			if(ptmp_buzz != NULL) 
				ptmp_buzz->freq = frequency;
			return 0;
					       
	  	case SP_IOC_VOL:
			if (copy_from_user(&volume, (void *)arg, sizeof(int)))
		   		return -EFAULT;
   			if(ptmp_buzz != NULL) 
				ptmp_buzz->volume = volume;
			return 0;
			
	  	case SP_IOC_STATUS: 
	    	return copy_to_user((void*)arg,&system_buzz[Number_iten],sizeof(BUZZER)) ? -EFAULT : 0;	
		
	  	case SP_IOC_NO:
			if (copy_from_user(&number, (void *)arg, sizeof(int)))
		    	return -EFAULT;
		 
	       	ptmp_buzz = &system_buzz[number];
			Number_iten = number;
	       	return 0;
			
		case SP_IOC_SHOW: 
	        if (copy_from_user(&times, (void *)arg, sizeof(int)))
		    	return -EFAULT;
	        if(ptmp_buzz != NULL)
			{
	        	ptmp_buzz->times = times;
				ptmp_buzz->buzz_off_time=10;
				memcpy(&want_buzz,ptmp_buzz,sizeof(BUZZER));
			}
			else 
				return 0;
			break;
			
	  	case SP_IOC_TIME:
	        if (copy_from_user(&buzz_on_time, (void *)arg, sizeof(int)))
		    	return -EFAULT;
	        if(ptmp_buzz != NULL) 
				ptmp_buzz->buzz_on_time = buzz_on_time;
	   		return 0;
		    
	  	case SP_IOC_STOP:
		    buzz_abort();
		    return 0;	    
	  
	  	case SP_IOC_USER_BUZZER:
  			buzz = &want_buzz;
	     	if (copy_from_user(buzz, (void *)arg, sizeof(BUZZER)))
				return -EFAULT;
			break;
			
	  	case SP_IOC_SYSTONE_BARCODE:
 			buzz = &system_buzz[BARCODE_READ];
			if( arg != 0)
			{
	        	if (copy_from_user(buzz, (void *)arg, sizeof(BUZZER)))
		  			return -EFAULT;
			}
   			memcpy(&want_buzz,&system_buzz[BARCODE_READ],sizeof(BUZZER));
			break;
			
	  	case SP_IOC_SYSTONE_KEYBOARD:
  			buzz = &system_buzz[KEYBOARD_IN];
			if( arg != 0)
			{
	        	if (copy_from_user(buzz, (void *)arg, sizeof(BUZZER)))
		  			return -EFAULT;
			}
   			memcpy(&want_buzz,&system_buzz[KEYBOARD_IN],sizeof(BUZZER));
			break;
	  
	  	case SP_IOC_SYSTONE_RECEIVE:
  			buzz = &system_buzz[DATA_RECEIVER];
			if( arg != 0)
			{
	        	if (copy_from_user(buzz, (void *)arg, sizeof(BUZZER)))
		  			return -EFAULT;
			}
   			memcpy(&want_buzz,&system_buzz[DATA_RECEIVER],sizeof(BUZZER));
			break;
	  	case SP_IOC_SYSTONE_ALARM:
  			buzz = &system_buzz[ALARM];
			if( arg != 0)
			{
	        	if (copy_from_user(buzz, (void *)arg, sizeof(BUZZER)))
		  			return -EFAULT;
			}
   			memcpy(&want_buzz,&system_buzz[ALARM],sizeof(BUZZER));
			break;
		
		case SP_IOC_OSE_BUZZER:
			if (copy_from_user(&ose_buzzer, (void *)arg, sizeof(OSE_BUZZER)))
				return -EFAULT;
			ose_buzzer_function();
			return 0;
			
		default:
			return -EFAULT;
			
	}
  	buzzer_sound();
  	return 0;
}


struct file_operations buzzer_speaker_fops = 
{
	read:		NULL,
	write:		NULL,
	ioctl:		sp_ioctl,
	open:		sp_open,
	release:	sp_close,

};

