/* vi: set sw=4 ts=4: */
/*
 *  UART(Universal Asynchronous Receiver/Transmitter Unit)
 *  For MIPS MR-2A chipset
 *
 *  9600BPS 8N1
 *
 *  Copyright (C) 2001-6-19 CIH <cih@coventive.com>
 *
 *  ChangeLog:
 *	2001-09-11 CIH <cih@coventive.com>
 *		Add turn on and turn off bar-code ioctl functions.
 *	2002-0520 Jeff <jeff_lee@coventive.com>
 *               Add for PHL4000 support	
 *      2002-0605 Sam <sam_liao@coventive.com>
 *              Sofeware De-bounce for scan button interrupt handler
 *      2002-0620 Jalen <jalen@coventive.com>
 *              barcode ISR,read rewrite
 *              Add modules Support
 *      2002-0809 Jeff <jeff_lee@coventive.com>
 *      	Add ioctl to accept command
 *		Disable barcode on before barcode device open
 *		GPIO20 -> SCAN key
 *		GPIO8  -> Module power (0->1 power on)
 *		GPIO26 -> Module reset (0 -> reset )
 *		GPIO28 -> Laser emission (1->0 Laser emission)
 */

#include <linux/kernel.h>

#include <linux/config.h>

#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/mm.h>
#include <linux/signal.h>
#include <linux/init.h>
#include <linux/delay.h>
#include <linux/random.h>
#include <linux/poll.h>
#include <linux/miscdevice.h>
//#include <linux/malloc.h>
#include <linux/bar_code.h>
#include <linux/module.h>
#include <linux/proc_fs.h>
#include <asm/vr4121/vr4121.h>

#include <asm/bitops.h>
#include <asm/uaccess.h>
#include <asm/irq.h>
#include <asm/system.h>

#include <asm/sp.h>

#include "Mr2H3.h"
#include "VrSys.h"

#define Barcode_VERSION  "2003-04-09 V1.3"
/* 0409 -> modify the write return value */
#define BAR_CODE_MAJOR				159
#define GPIO0_INTERRUPT_NUMBER		40

#define EMISSION_ON			1
#define EMISSION_OFF		0

#define POWER_ON			1
#define POWER_OFF			0

#define DEFAULT_KEY_DELAY			3

static DECLARE_WAIT_QUEUE_HEAD(bar_code_queue);

unsigned long flags;

#define CIRBUFSIZE	4096
#define SAFETYSIZE	256
static unsigned char cirbuf[CIRBUFSIZE];
static int emptyhead = 0;
static int unreadhead = 0;
unsigned char Command[80];
int BARCODE_OPEN = 0;
int BARCODE_BUSY=0;
extern int KEYMODE;
static volatile int scanner_status = 0;
static struct timer_list KeyOutTimer,ScannerOnTimer;
static int key_delay = DEFAULT_KEY_DELAY;
static int save_status;

/*
 * Pre- and suffix for keyboard emulation.
 */
static char prefix[17], suffix[17];

//#define SOUND_OUTPUT

#if (CONFIG_PHL4000 || CONFIG_PHL4100)
#ifdef CONFIG_PHL_4000
unsigned int GPIO20_BIT;
#endif
static void Barcode_on(void);
static void Barcode_off(void);

#ifdef SOUND_OUTPUT
extern int VR4121_SP_reg_config(void);
extern void sound_PHL4000_barcode_on(void);
extern void MR2A_PGU_reg_config_init(void);
extern void sound_PHL4000_barcode_off(void);
#endif
/* Jeff 5/31 */
void Command_DisableTimeout(void);

/* For buzzer */
#endif

int __init bar_code_setup(void);
void ScannerOn(void);
static void ScannerOff(void);
extern int pm_access_device(void);
static void bar_code_handler(int irq, void *dev_id, struct pt_regs *regs);
static long long bar_code_lseek(struct file *file, long long offset, int orig);
static unsigned int bar_code_poll(struct file *filp, poll_table *wait);
static int bar_code_open(struct inode *inode, struct file *filp);
static int bar_code_release(struct inode *inode, struct file *filp);
static int bar_code_fasync(int fd, struct file *filp, int on);
static ssize_t bar_code_read(struct file *file,char *buffer,size_t length,loff_t *offset);
static ssize_t bar_code_write(struct file *file,const char *buffer,size_t length,loff_t *offset);
static int bar_code_ioctl(struct inode * inode,struct file * file,unsigned int cmd,unsigned long arg);
static void arch_bar_code_initial(void);
static void F65_initial(void);
static void F65R_Ret(void);
static void Uart_clr(void);
static void ContIntAsic(unsigned long block, unsigned long en);
static void ContIntPeri(unsigned long block, unsigned long en);
static void Laser_Emission( int status );
static void Module_Power( int status );
static void Module_Reset(void);

//static unsigned long wait_count;

#ifdef CONFIG_PM_SLEEP
void pm_barcode_init(void)
{
   F65_initial();
   F65R_Ret();
}
#endif

static void Laser_Emission( int status )
{
	*( ( volatile unsigned short *)0xab000102 ) |= 0x1000;
		
	if(status == EMISSION_ON)
		*( ( volatile unsigned short *)0xab000106 ) &= ~0x1000;
	else
		*( ( volatile unsigned short *)0xab000106 ) |= 0x1000;
}	

static void Module_Power( int status )
{
	*( ( volatile unsigned short *)0xab000100 ) |= 0x0100;

	if(status == POWER_ON)
		*( ( volatile unsigned short *)0xab000104 ) |= 0x0100;
	else
		*( ( volatile unsigned short *)0xab000104 ) &= ~0x0100;
}

static void Module_Reset(void)
{
	*( ( volatile unsigned short *)0xab000102 ) |= 0x0400;
	
	*( ( volatile unsigned short *)0xab000106 ) |= 0x0400;
	udelay(100);
	*( ( volatile unsigned short *)0xab000106 ) &= ~0x0400;
	udelay(100);
	*( ( volatile unsigned short *)0xab000106 ) |= 0x0400;		
}
	
static void arch_bar_code_initial(void)
{
    F65_initial();
    F65R_Ret();
}

static void F65_initial(void)
{
    unsigned char data;

    /* Supply UART clock */
    *( volatile unsigned short *)0xAA000262 = 0x0020;
    udelay(100);	/* Jeff */
    *( volatile unsigned short *)0xAA000262 = 0x0000;

    data = *(volatile unsigned char *)(F65SIU+0x0002);
    data = *(volatile unsigned char *)(F65SIU+0x0004);
    data = *(volatile unsigned char *)(F65SIU+0x0006);
    data = *(volatile unsigned char *)(F65SIU+0x0008);
    data = *(volatile unsigned char *)(F65SIU+0x000a);

    *((unsigned short *)0xab000100) |= 0x4000;/* GPIO14 output */
    *((unsigned short *)0xab000104) &= 0xbfff;/* GPIO14 = Low */

    /* turn on barcode power and reset barcode */
    /* Set GPIO26 to be input mode */
    *((volatile unsigned short *)0xab000102) &= ~0x0400;
    /* GPIO8 = LOW*/
    Module_Power(POWER_OFF);

    /* GPIO28 = HIGH */
    Laser_Emission(EMISSION_OFF);

    udelay(100);
    /* GPIO8 = HIGH */
    Module_Power(POWER_ON);
    mdelay(15);

#ifdef CONFIG_PHL4000
    /* GPIO 20 as interrupt pin */
    *(unsigned short *)( GIUIOSELH )    &= 0xffef;
    *(unsigned short *)( GIUINTTYPH )   |= 0x0010 ;
    *(unsigned short *)( GIUINTENH )    |= 0x0010 ;
    *(unsigned short *)( GIUINTHTSELH ) |= 0x0010 ;
    *(unsigned short *)( GIUINTSTATH )  |= 0x0010 ;
    /* initial the buzzer */
#ifdef SOUND_OUTPUT
    VR4121_SP_reg_config();
    MR2A_PGU_reg_config_init();
    sound_PHL4000_barcode_off();
#endif
    /* unmask GPIO20 pin */
    *(unsigned short *)( MGIUINTHREG ) |= 0x0010;
    *(unsigned short *)( MSYSINT1REG ) |= 0x0100;
#endif
}

static void SetCommParms(int status)
{
	int rate;

	save_status = status;
    rate = status >> 5;
    *(volatile unsigned char *)(F65SIU + 0x0006) = 0x80;  /* Set divider latch */

    *( unsigned char *)(F65SIU+0x0000) = (rate & 0xff);
    *( unsigned char *)(F65SIU+0x0002) = (rate >> 8) & 0xff;
    
    *(volatile unsigned char *)(F65SIU+0x0006) = status & 0x1f;
	Uart_clr();
}

static void F65R_Ret(void)
{
    unsigned char data;

	SetCommParms(0x183);				// 9600,n,8,1  Divisor = 0x000C, modem status = 0x03

    /* Set FIFO */
    *( unsigned char *)( F65SIU + 0x0004 ) = 0x00;
    data = *(unsigned char *)(F65SIU+0x0004);
    
    *( unsigned char *)( F65SIU + 0x0004 ) =  0xc1;
//	*(unsigned char *)(F65SIU+0x0004) = 0x81;
    *( unsigned char *)( F65SIU + 0x000E ) =  0x81;
    *( unsigned char *)( F65SIU + 0x0002 ) |= 0x01;
//    *( unsigned char *)( F65SIU + 0x0002 ) |= 0x04;
    ContIntAsic(MASKUARTASIC, INT_ON);

    *( unsigned char *)( F65SIU + 0x0008 ) |= 0x03;
}

static void Uart_clr(void)
{
    unsigned char data;

    data = *(unsigned char *)(F65SIU+0x0000);
    data = *(unsigned char *)(F65SIU+0x0004);
    data = *(unsigned char *)(F65SIU+0x000a);
    data = *(unsigned char *)(F65SIU+0x000c);
}

static void ContIntAsic(unsigned long block, unsigned long en)
{
  unsigned short *int_mask_data;
  unsigned short data_buf;

  int_mask_data = (unsigned short *)(MR2ICU+0x04);
  data_buf = *int_mask_data;

  switch( block )
  {
  case MASKUARTASIC:
    data_buf = ( data_buf & 0xffdf ) | (unsigned short)( en << 5 );
    *int_mask_data = data_buf;
    break;

  default:
	  break;
  }

  data_buf = *(unsigned short *)(MGIUINTLREG);
  data_buf = ( data_buf & 0xfffe ) | (unsigned short)( en << 0 );
  *(unsigned short *)(MGIUINTLREG) = data_buf;

  ContIntPeri( MASKGIU, en );
}

static void ContIntPeri(unsigned long block, unsigned long en)
{
	unsigned short  data_buf;

	switch( block )
  	{
  		case MASKGIU:
    		data_buf = *(unsigned short *)( MSYSINT1REG );
		    data_buf = ( data_buf & 0xfeff ) | (unsigned short)( en << 8 );
		    *(unsigned short *)( MSYSINT1REG ) = data_buf;
		    break;

		default:
    		break;
	}
}

void Barcode_on(void)
{
	char tmp;

   	*(volatile unsigned char *)(F65SIU+0x0000) = 0x1b;
   	mdelay(1);
   	*(volatile unsigned char *)(F65SIU+0x0000) = 0x50;	
   	mdelay(1);
   	*(volatile unsigned char *)(F65SIU+0x0000) = 0x4b;
	mdelay(1);
   	*(volatile unsigned char *)(F65SIU+0x0000) = 0x0d;
   	mdelay(1);
	
   	// clear read buffer
   	while ( (*(unsigned char *)(F65SIU+0x000A) & 0x0001) == 0x0001 )
       	tmp = *(unsigned char *)(F65SIU+0x0000);
	Laser_Emission(EMISSION_ON);
}

void Command_DisableTimeout(void)
{
    *(volatile unsigned char *)(F65SIU+0x0000) = 0x1b;
    mdelay(1);
    *(volatile unsigned char *)(F65SIU+0x0000) = 0x59;
    mdelay(1);
    *(volatile unsigned char *)(F65SIU+0x0000) = 0x30;
    mdelay(1);
    *(volatile unsigned char *)(F65SIU+0x0000) = 0x0d;
    mdelay(1);
}

void Barcode_off(void)
{
    *(volatile unsigned char *)(F65SIU+0x0000) = 0x1b;
    mdelay(1);
    *(volatile unsigned char *)(F65SIU+0x0000) = 0x50;
    mdelay(1);
    *(volatile unsigned char *)(F65SIU+0x0000) = 0x4c;
    mdelay(1);
    *(volatile unsigned char *)(F65SIU+0x0000) = 0x0d;
    mdelay(1);
	Laser_Emission(EMISSION_OFF);
}

static void ScannerOff(void)
{
	scanner_status = 0;
	*(volatile unsigned char *)(F65SIU+0x0000) = 0x1b;  //ESC
    mdelay(1);
    *(volatile unsigned char *)(F65SIU+0x0000) = 'V';
    mdelay(1);
    *(volatile unsigned char *)(F65SIU+0x0000) = 0x0d;  //CR
	del_timer(&ScannerOnTimer);
}
                                                                                                                        
void ScannerOn(void)
{
	if (scanner_status != 0)
		return;
	scanner_status = 1;

    *(volatile unsigned char *)(F65SIU+0x0000) = 0x1b;  //ESC
    mdelay(1);
    *(volatile unsigned char *)(F65SIU+0x0000) = 'Z';
    mdelay(1);
    *(volatile unsigned char *)(F65SIU+0x0000) = 0x0d;  //CR
	
	mod_timer(&ScannerOnTimer,jiffies + 4 * 100); 
}

void KeyOut(void)
{
	int c;
   	extern void PushKey(int key);

	c=cirbuf[emptyhead];
	if (c==0)
	{
		scanner_status = 0;
		del_timer(&KeyOutTimer);
	}
	else
	{	PushKey(c);
		mod_timer(&KeyOutTimer,jiffies + key_delay);
		emptyhead++;
	}
}

static void bar_code_handler(int irq, void *dev_id, struct pt_regs *regs)
{
   unsigned char c;
   static int state;
   static int len;
   static int long_codelength;
	
 	if (KEYMODE == KEY_AUTO)
	{
		while ( (*(unsigned char *)(F65SIU+0x000A) & 0x0001) == 0x0001 )
      	{
			c = *(unsigned char *)(F65SIU+0x0000);

       		switch(state) 
			{
         		case 0:
            		if ((c == 0x01) || (c == 0x02))
                		state = 1;
					long_codelength = c == 0x01;
                	break;

          		case 1:					// Symbology
					state = 2;
					break;

		  		case 2:                 // Bar code length
                	len = c - '0';
					if (!long_codelength)
						state = 7;
					break;

                case 3:
                case 4:
                case 5:
                case 6:
                	len *= 10;
                	len +=  c - '0';
					state++;
					break;

	          	case 7:					// Last char of bar code length
                	len *= 10;
                	len += c - '0';
	                state = 8;
                	emptyhead = 0;
 			       	del_timer(&ScannerOnTimer);
					{ // Send prefix
						char* p = prefix;
						while (*p)
						{
							cirbuf[emptyhead++] = *p++;
							if (emptyhead == CIRBUFSIZE)
								emptyhead = 0;
						}
					}
                 	break;
					   
		  		case 8:
                	cirbuf[emptyhead++]= c;
	                if (emptyhead >= len)   
					{	// Send suffix
						char* s = suffix;
						while (*s)
						{
							cirbuf[emptyhead++]= *s++;
							if (emptyhead == CIRBUFSIZE)
								emptyhead = 0;
						}
		               	cirbuf[emptyhead]= '\0';
						emptyhead = 0;
   						mod_timer(&KeyOutTimer, jiffies + key_delay);
						state = 0;
					}
                	break; 
			}
		}
	}
   	else
   	{
    	while ( (*(unsigned char *)(F65SIU+0x000A) & 0x0001) == 0x0001 )
    	{
			c = *(unsigned char *)(F65SIU+0x0000);
			cirbuf[emptyhead++] = c;
    		if (emptyhead == CIRBUFSIZE)
        		emptyhead = 0;
		}
	}
}

static long long bar_code_lseek(struct file *file,  long long offset, int orig)
{
    return -ESPIPE;
}

static unsigned int bar_code_poll(struct file *filp, poll_table *wait)
{
    unsigned int mask = 0;

    poll_wait(filp, &bar_code_queue, wait);

    if (emptyhead != unreadhead) mask |= POLLIN | POLLRDNORM;

    return mask;
}


static int bar_code_open(struct inode *inode, struct file *filp)
{
   	int i;


   	BARCODE_OPEN = 1;
   	emptyhead = unreadhead = 0;

   	/* clear circular buffer */
	for(i=0; i<CIRBUFSIZE; i++)
		cirbuf[i] = 0x00;

   	MOD_INC_USE_COUNT;
   	return 0;
}

static int bar_code_release(struct inode *inode, struct file *filp)
{
	MOD_DEC_USE_COUNT;
   	BARCODE_OPEN = 0;
   	return 0;
}

static int bar_code_fasync(int fd, struct file *filp, int on)
{
   	return 0;
}

static ssize_t bar_code_read(struct file *file,char *buffer,size_t length,loff_t *offset)
{
	int i;

	for(i=0;i<length;i++)
	{
        if (unreadhead == emptyhead)
            break;
		put_user(cirbuf[unreadhead++],buffer + i);

		if (unreadhead == CIRBUFSIZE)
			unreadhead = 0;
	}	
    return i;
}

static ssize_t bar_code_write(struct file *file,const char *buffer,size_t length,loff_t *offset)
{
	int i,c;

	for(i = 0;i < length; i++)
	{
		get_user(c, buffer + i);
	    *(volatile unsigned char *)(F65SIU+0x0000) = c;
	    mdelay(1);
	}
  	return length;
}

static int bar_code_ioctl(struct inode * inode,struct file * file,unsigned int cmd,unsigned long arg)
{
    unsigned int baud_status;
    int  i;
    int  tmp_status;
	extern void PushKey(int key);

    switch ( cmd )
    {
        case BAR_CODE_ON:
	 		Barcode_on();
    		break;

        case BAR_CODE_OFF:
			Barcode_off();
	    	break;

        case SET_KEY_MODE:
            if (copy_from_user(&KEYMODE, (void *)arg, sizeof(KEYMODE)))
                return -EFAULT;
			break;

		case SET_LASER_EMITTER:
		    if (copy_from_user(&tmp_status, (void *)arg, sizeof(tmp_status)))
				return -EFAULT;
	    
		    if(tmp_status) 
				Laser_Emission(EMISSION_ON);
		    else 
				Laser_Emission(EMISSION_OFF);
	    	break;
	
		case SET_MODULE_POWER:
		    if (copy_from_user(&tmp_status, (void *)arg, sizeof(tmp_status)))
        		return -EFAULT;
	    
            if(tmp_status) 
				Module_Power(POWER_ON);
	    	else 
				Module_Power(POWER_OFF);
	    	break;
		    
		case BAR_CODE_CLEAR:
		    emptyhead = unreadhead = 0;
			break;

        case BAR_CODE_RECV_SIZE:
 			i = CIRBUFSIZE + emptyhead - unreadhead;
			if (i >= CIRBUFSIZE)
				i -= CIRBUFSIZE;
            if(copy_to_user((void*)arg, &i, sizeof(i)))
                return -EFAULT;
            break;
                                                                                                                          
		case SET_BAUD:
		    if (copy_from_user(&baud_status, (void *)arg, sizeof(baud_status)))
        		return -EFAULT;

            /*---------------------------------------------------------------
		     * baud_status -->
		     * bit[1:0] -> 	The length of character,
	    	 *				11: 8bits, 10: 7bits, 01: 6bits, 00: 5bits
		     * bit2     -> Stop bit setting
		     *       		 0: 1bits, 1: 1.5bits
	    	 * bit3     -> Parity enable
		     *       		 0: No parity, 1: enable parity
		     * bit4     -> Parity setting
	    	 *       		 0: Odd parity, 1: Even parity
		     * bit[20:5]-> Baudrate divider
			 * 				 Baudrate = 18.432 MHz / (160 * divider)
		     * 				 divider  = 18.432 MHz / (160 * baudrate)
			 * 
	    	 *---------------------------------------------------------------*/
	   		SetCommParms(baud_status); 
		    break;
		
		case GET_BAUD:
           if(copy_to_user((void*)arg, &save_status, sizeof(save_status)))
                return -EFAULT;
			break; 			
	
		case MODULE_RESET:
			Module_Reset();
			break;

		case SET_PREFIX:
		    if (copy_from_user(&prefix, (void *)arg, sizeof prefix))
        		return -EFAULT;
			prefix[16] = 0; // Make sure it is zero terminated
			break;

		case SET_SUFFIX:
		    if (copy_from_user(&suffix, (void *)arg, sizeof suffix))
        		return -EFAULT;
			suffix[16] = 0; // Make sure it is zero terminated
			break;

        case GET_KEY_DELAY:
			if (copy_to_user((void*)arg, &key_delay, sizeof key_delay))
				return -EFAULT;
			break;

		case SET_KEY_DELAY:
			if (copy_from_user(&key_delay, (void*)arg, sizeof key_delay))
				return -EFAULT;
			break;

		default:
	    	return -EINVAL;
    }

    return 0;
}

struct file_operations bar_code_fops = 
{
    llseek:	bar_code_lseek,
    read: 	bar_code_read,
    write:	bar_code_write,
    poll: 	bar_code_poll,
    ioctl:	bar_code_ioctl,
    open: 	bar_code_open,
    release:	bar_code_release,
    fasync:	bar_code_fasync
};

static int print_visible(char* dst, char* src)
{
	char* orig = dst;
	int i;

	for (i=0; (i<17) && (*src); i++)
	{
		if (*src >= 0)
		{
			if (*src < 0x20)
			{
				*dst++ = '^';
				*dst++ = (*src) + '@';
			}
			else
			{
				*dst++ = *src;
			}
		}
		else
		{
			dst += sprintf(dst, "\\x%02X", (unsigned char)*src);
		}

		src++;
	}

	return dst - orig;
}

static int string_length(char* str)
{
	int l = 0;

	while(*str++) l++;

	return l;
}

static int bar_code_read_proc(char* buf, char** start, off_t offset,
							int len, int* eof, void* private)
{
	int slen = 0;
	int rate = (save_status >> 5);
	
	if (rate == 0) rate = 0x10000;

	slen  = sprintf(buf + slen, "Communication settings: %d,%c,%d,%d\n",
		1843200 / (rate << 4),
		(save_status & 0x08) ? ((save_status & 0x10)?'E':'O') : 'N',
		5 + (save_status & 0x03),
		1 + ((save_status >> 2) & 0x01));

	slen += sprintf(buf + slen, "Prefix(%d): ", string_length(prefix));
	slen += print_visible(buf + slen, prefix);
	slen += sprintf(buf + slen, "\nSuffix(%d): ", string_length(suffix));
	slen += print_visible(buf + slen, suffix);
	slen += sprintf(buf + slen, "\n");

	slen += sprintf(buf + slen, "Inter-character delay: %d\n", key_delay);
	return slen;
}

int __init bar_code_setup(void)
{
   register_chrdev(BAR_CODE_MAJOR, "bar_code", &bar_code_fops);

   printk(KERN_INFO "Barcode Driver v" Barcode_VERSION "\n");
   
   request_irq(GPIO0_INTERRUPT_NUMBER, bar_code_handler, SA_SHIRQ, "bar_code", NULL);
   arch_bar_code_initial();

   // Initialize the pre- and suffix
   prefix[0] = '\0';
   suffix[0] = '\r'; suffix[1] = '\0';

   key_delay = DEFAULT_KEY_DELAY;

   init_timer(&KeyOutTimer);
   KeyOutTimer.function = ( void (*)(unsigned long) ) KeyOut;
   init_timer(&ScannerOnTimer);
   ScannerOnTimer.function = ( void (*)(unsigned long) ) ScannerOff;

   // Create /proc entries
   create_proc_read_entry("bar_code", 0, 0, bar_code_read_proc, NULL);

   return 0;
}

void __exit bar_code_exit(void)
{
   while(MOD_IN_USE){
      MOD_DEC_USE_COUNT;
   }
   BARCODE_OPEN = 0; 
   free_irq(GPIO0_INTERRUPT_NUMBER, NULL);
   unregister_chrdev(BAR_CODE_MAJOR,"bar_code");
}

