/* vi: set sw=4 ts=4: */
/*
 *  UART(Universal Asynchronous Receiver/Transmitter Unit)
 *  For MIPS MR-2A chipset
 *
 */

#include <linux/config.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/mm.h>
#include <linux/signal.h>
#include <linux/init.h>
#include <linux/delay.h>
#include <linux/random.h>
#include <linux/poll.h>
#include <linux/miscdevice.h>
#include <linux/zeevo_blt.h>
#include <linux/module.h>
#include <asm/vr4121/vr4121.h>

#include <asm/bitops.h>
#include <asm/uaccess.h>
#include <asm/irq.h>
#include <asm/system.h>
#include <asm/sp.h>

#include "Mr2H3.h"
#include "VrSys.h"

#define VERSION  					"2004-02-03 V0.0"
#define BLUETOOTH_MAJOR				160
#define GPIO0_INTERRUPT_NUMBER		40

static DECLARE_WAIT_QUEUE_HEAD(bluetooth_queue);

unsigned long flags;

#define CIRBUFSIZE	1000
static unsigned char cirbuf[CIRBUFSIZE];
static int emptyhead = 0;
static int unreadhead = 0;
static int save_status;
static int driver_open;
static wait_queue_head_t wait;

int __init bluetooth_setup(void);
extern int pm_access_device(void);
static void bluetooth_handler(int irq, void *dev_id, struct pt_regs *regs);
static int bluetooth_open(struct inode *inode, struct file *filp);
static int bluetooth_release(struct inode *inode, struct file *filp);
static ssize_t bluetooth_read(struct file *file,char *buffer,size_t length,loff_t *offset);
static ssize_t bluetooth_write(struct file *file,const char *buffer,size_t length,loff_t *offset);
static int bluetooth_ioctl(struct inode * inode,struct file * file,unsigned int cmd,unsigned long arg);
static void arch_bluetooth_initial(void);
static void F65_initial(void);
static void F65R_Ret(void);
static void Uart_clr(void);
static void ContIntAsic(unsigned long en);
static void Module_Power( int status );


#ifdef CONFIG_PM_SLEEP
void pm_bluetooth_init(void)
{
   F65_initial();
   F65R_Ret();
}
#endif

static void Module_Power( int status )
{
	*( ( volatile unsigned short *)GIUIOSELL ) |= 0x0100;

	if(status == POWER_ON)
		*( ( volatile unsigned short *)GIUPIODL ) |= 0x0100;
	else
		*( ( volatile unsigned short *)GIUPIODL ) &= ~0x0100;
}

static void arch_bluetooth_initial(void)
{
    F65_initial();
    F65R_Ret();
}

static void F65_initial(void)
{
    unsigned char data;

    /* Supply UART clock */
    *( volatile unsigned short *)0xAA000262 = 0x0020;
    udelay(100);
    *( volatile unsigned short *)0xAA000262 = 0x0000;

    data = *(volatile unsigned char *)(F65SIU+0x0002);
    data = *(volatile unsigned char *)(F65SIU+0x0004);
    data = *(volatile unsigned char *)(F65SIU+0x0006);
    data = *(volatile unsigned char *)(F65SIU+0x0008);
    data = *(volatile unsigned char *)(F65SIU+0x000a);

    *((unsigned short *)GIUIOSELL) |= 0x4000;			// GPIO14 output 
    *((unsigned short *)GIUPIODL) &= 0xbfff;			// GPIO14 = Low 

    /* Set GPIO26 to be output mode, since this pin is not used by the bluetooth driver (Reset for barcode ASIC) */
    *((volatile unsigned short *)GIUIOSELH) |= 0x0400;
    
	Module_Power(POWER_OFF);

    /* Set GPIO28 to be output mode, since this pin is not used by the bluetooth driver (Trigger barcode ASIC) */
	*( ( volatile unsigned short *)GIUIOSELH) |= 0x1000;

//    udelay(100);
    
//	Module_Power(POWER_ON);
//    mdelay(15);
}
/*
 * Set comm. params according to status:
	
	bit[1:0] ->  The length of character,
    	11: 8bits, 10: 7bits, 01: 6bits, 00: 5bits
	bit2     -> Stop bit setting
		0: 1bits, 1: 1.5bits
	bit3     -> Parity enable
		0: No parity, 1: enable parity
	bit4     -> Parity setting
		0: Odd parity, 1: Even parity
	bit[20:5]-> Baudrate 
		Baudrate = 18.432 MHz / (160 * divider)

    	0x01 means 115200 baud
    	0x02 means 57600 baud
	    0x03 means 38400 baud
	    0x06 means 19200 baud
	    0x0C means 9600 baud
*/
static void SetCommParms(int status)
{
	int rate;

	save_status = status;
    rate = status >> 5;
    *(volatile unsigned char *)(F65SIU + 0x0006) = 0x80;  /* Set divider latch */
                                                                                                                                         
    *( unsigned char *)(F65SIU+0x0000) = (rate & 0xff);
    *( unsigned char *)(F65SIU+0x0002) = (rate >> 8) & 0xff;
                                                                                                                                         
    *(volatile unsigned char *)(F65SIU+0x0006) = status & 0x1f;
	Uart_clr();
}

static void F65R_Ret(void)
{
    unsigned char data;

	SetCommParms(0x0023);				// 115200,n,8,1     

    /* Set FIFO */
    *( unsigned char *)( F65SIU + 0x0004 ) = 0x00;
    data = *(unsigned char *)(F65SIU+0x0004);
    
    *( unsigned char *)( F65SIU + 0x0004 ) =  0xc1;
    *( unsigned char *)( F65SIU + 0x000E ) =  0x81;
    *( unsigned char *)( F65SIU + 0x0002 ) |= 0x01;

    ContIntAsic(INT_ON);
    *( unsigned char *)( F65SIU + 0x0008 ) |= 0x03;
}

static void Uart_clr(void)
{
    unsigned char data;

    data = *(unsigned char *)(F65SIU+0x0000);
    data = *(unsigned char *)(F65SIU+0x0004);
    data = *(unsigned char *)(F65SIU+0x000a);
    data = *(unsigned char *)(F65SIU+0x000c);
}

static void ContIntAsic(unsigned long en)
{
	unsigned short *int_mask_data;
  	unsigned short data_buf;

  	int_mask_data = (unsigned short *)(MR2ICU+0x04);
  	data_buf = *int_mask_data;

	data_buf = ( data_buf & 0xffdf ) | (unsigned short)( en << 5 );
	*int_mask_data = data_buf;

  	data_buf = *(unsigned short *)(MGIUINTLREG);
  	data_buf = ( data_buf & 0xfffe ) | (unsigned short)( en << 0 );
  	*(unsigned short *)(MGIUINTLREG) = data_buf;

	data_buf = *(unsigned short *)( MSYSINT1REG );
    data_buf = ( data_buf & 0xfeff ) | (unsigned short)( en << 8 );
    *(unsigned short *)( MSYSINT1REG ) = data_buf;
}

static void bluetooth_handler(int irq, void *dev_id, struct pt_regs *regs)
{
   	unsigned char c;
	
	while ( (*(unsigned char *)(F65SIU+0x000A) & 0x0001) == 0x0001)
   	{
		c = *(unsigned char *)(F65SIU+0x0000);
		cirbuf[emptyhead++] = c;
   		if (emptyhead == CIRBUFSIZE)
       		emptyhead = 0;
	}
}

static int bluetooth_open(struct inode *inode, struct file *filp)
{
    printk(KERN_INFO "Bluetooth open\n");

   	driver_open = 1;
   	emptyhead = unreadhead = 0;

   	MOD_INC_USE_COUNT;
   	return 0;
}

static int bluetooth_release(struct inode *inode, struct file *filp)
{
    printk(KERN_INFO "Bluetooth release\n");

	MOD_DEC_USE_COUNT;
   	driver_open = 0;
   	return 0;
}

static ssize_t bluetooth_read(struct file *file,char *buffer,size_t length,loff_t *offset)
{
	int i;

    printk(KERN_INFO "Bluetooth read\n");

	for(i=0;i<length;i++)
	{
        if (unreadhead == emptyhead)
            break;
		
		put_user(cirbuf[unreadhead++],buffer + i);

		if (unreadhead == CIRBUFSIZE)
			unreadhead = 0;
	}	
    return i;
}

static ssize_t bluetooth_write(struct file *file,const char *buffer,size_t length,loff_t *offset)
{
	int i,c;

    printk(KERN_INFO "Bluetooth write\n");

	for(i = 0;i < length; i++)
	{
		get_user(c, buffer + i);
	    *(volatile unsigned char *)(F65SIU+0x0000) = c;
	    mdelay(1);
	}
  	return length;
}

static int bluetooth_ioctl(struct inode * inode,struct file * file,unsigned int cmd,unsigned long arg)
{
    unsigned int baud_status;
    int  i;
    int  tmp_status;
	DEBUG_STRUCT debug_struct;

	printk(KERN_INFO "Bluetooth ioctl\n");

    switch ( cmd )
    {
		case BLT_POWER:
		    if (copy_from_user(&tmp_status, (void *)arg, sizeof(tmp_status)))
        		return -EFAULT;
	    
            if(tmp_status) 
				Module_Power(POWER_ON);
	    	else 
				Module_Power(POWER_OFF);
	    	break;
		    
		case BLT_CLEAR:
		    emptyhead = unreadhead = 0;
			break;

        case BLT_SIZE:
 			i = CIRBUFSIZE + emptyhead - unreadhead;
			if (i >= CIRBUFSIZE)
				i -= CIRBUFSIZE;
            if(copy_to_user((void*)arg, &i, sizeof(i)))
                return -EFAULT;
            break;

		case BLT_SET_BAUD:
		    if (copy_from_user(&baud_status, (void *)arg, sizeof(baud_status)))
        		return -EFAULT;
	   		SetCommParms(baud_status); 
		    break;
		
		case BLT_GET_BAUD:
           if(copy_to_user((void*)arg, &save_status, sizeof(save_status)))
                return -EFAULT;
			break; 			
		case BLT_GET_PORT:
            if (copy_from_user(&debug_struct, (void *)arg, sizeof(PORT_STRUCT)))
                return -EFAULT;
			*(unsigned char *)(F65SIU+debug_struct.offset)=debug_struct.value;
			break;
        case BLT_SET_PORT:
            if (copy_from_user(&debug_struct, (void *)arg, sizeof(PORT_STRUCT)))
                return -EFAULT;
			
			debug_struct.value = *(unsigned char *)(F65SIU+debug_struct.offset);
           
			if (copy_to_user((void*)arg,&debug_struct,sizeof(PORT_STRUCT)))
                return -EFAULT;
            break;
		case BLT_RTS_ON:
			*(unsigned char *)(F65SIU+8) |= 0x02;
			break;
        case BLT_RTS_OFF:
            *(unsigned char *)(F65SIU+8) &= ~0x02;
            break;
		default:
	    	return -EINVAL;
    }

    return 0;
}

struct file_operations bluetooth_fops = 
{
    read: 		bluetooth_read,
    write:		bluetooth_write,
    ioctl:		bluetooth_ioctl,
    open: 		bluetooth_open,
    release:	bluetooth_release,
};

int __init bluetooth_setup(void)
{
   register_chrdev(BLUETOOTH_MAJOR, "bluetooth", &bluetooth_fops);

   printk(KERN_INFO "Bluetooth driver v" VERSION "\n");
   
   request_irq(GPIO0_INTERRUPT_NUMBER, bluetooth_handler, SA_SHIRQ, "bluetooth", &wait);
   arch_bluetooth_initial();
   return 0;
}

void __exit bluetooth_exit(void)
{
	while(MOD_IN_USE)
	    MOD_DEC_USE_COUNT;
   	driver_open = 0; 
   	free_irq(GPIO0_INTERRUPT_NUMBER, NULL);
   	unregister_chrdev(BLUETOOTH_MAJOR,"bluetooth");
}

