/*
 *	pnc2000.c - mapper for Photron PNC-2000 board.
 *
 * Copyright (C) 2000 Crossnet Co. <info@crossnet.co.jp>
 *
 * This code is GPL
 *
 * $Id: minigateway-flash.c,v 1.2 2004/07/16 09:06:11 jbo Exp $
 * 
 * add support minigateway by smills_ho@coventive.com
 */

#include <linux/module.h>
#include <linux/types.h>
#include <linux/kernel.h>

#include <linux/mtd/mtd.h>
#include <linux/mtd/map.h>
#include <linux/mtd/partitions.h>
#include <asm/vr4122.h>

#define WINDOW_ADDR 0xbf000000
#define WINDOW_SIZE (0x00800000*2)

/* 
 * MAP DRIVER STUFF
 */

__u8 pnc_read8(struct map_info *map, unsigned long ofs)
{
  return *(__u8 *)(WINDOW_ADDR + ofs);
}

__u16 pnc_read16(struct map_info *map, unsigned long ofs)
{
  return *(__u16 *)(WINDOW_ADDR + ofs);
}

__u32 pnc_read32(struct map_info *map, unsigned long ofs)
{
  return *(volatile unsigned int *)(WINDOW_ADDR + ofs);
}

void pnc_copy_from(struct map_info *map, void *to, unsigned long from, ssize_t len)
{
  memcpy(to, (void *)(WINDOW_ADDR + from), len);
}

void pnc_write8(struct map_info *map, __u8 d, unsigned long adr)
{
  *(__u8 *)(WINDOW_ADDR + adr) = d;
}

void pnc_write16(struct map_info *map, __u16 d, unsigned long adr)
{
  *(__u16 *)(WINDOW_ADDR + adr) = d;
}

void pnc_write32(struct map_info *map, __u32 d, unsigned long adr)
{
  *(__u32 *)(WINDOW_ADDR + adr) = d;
}

void pnc_copy_to(struct map_info *map, unsigned long to, const void *from, ssize_t len)
{
  memcpy((void *)(WINDOW_ADDR + to), from, len);
}

struct map_info pnc_map = {
#ifdef CONFIG_MINIGATEWAY   
	name: "MiniGateway",
#else
        name: "CAPCELLA",
#endif     
	size: WINDOW_SIZE,
	buswidth: 4,
	read8: pnc_read8,
	read16: pnc_read16,
	read32: pnc_read32,
	copy_from: pnc_copy_from,
	write8: pnc_write8,
	write16: pnc_write16,
	write32: pnc_write32,
	copy_to: pnc_copy_to
};


/*
 * MTD 'PARTITIONING' STUFF 
 */
static struct mtd_partition pnc_partitions[3] = {
/*	{
		name: "etc",
		size:   0x001e0000,
		offset: 0x00020000
	},
	{
		name: "usr",
		size:   0x00a00000,
		offset: 0x00100000
	},*/
     {
	  name: "root",
	  size:   0x00be0000,
	  offset: 0x00020000
     },
/*	{
		name: "bootloader",
		size:   0x00020000,
		offset: 0x00c00000
	},*/
        {
	        name: "kernel",
	        size:   0x001e0000,
	        offset: 0x00c20000
	},
        {
	        name: "etc",
	        size:   0x001e0000,
	        offset: 0x00e00000
	}
};

/* 
 * This is the master MTD device for which all the others are just
 * auto-relocating aliases.
 */
static struct mtd_info *mymtd;

int __init init_pnc2000(void)
{
	printk(KERN_NOTICE "Flash mapping: %x at %x\n", WINDOW_SIZE, WINDOW_ADDR);

        *VR41XX_GIUIOSELL |= 0x0a00;
        *VR41XX_GIUIOSELL |= 0x0a00;
       /* Enable flash read/write cycle */
        *(__u16 *)( VR41XX_BCUCNTREG1 )   |= 0x0050;
   
	mymtd = do_map_probe("cfi_probe", &pnc_map);
	if (mymtd) {
		mymtd->module = THIS_MODULE;
		return add_mtd_partitions(mymtd, pnc_partitions, 3);
	}

	return -ENXIO;
}

static void __exit cleanup_pnc2000(void)
{
	if (mymtd) {
		del_mtd_partitions(mymtd);
		map_destroy(mymtd);
	}
}

module_init(init_pnc2000);
module_exit(cleanup_pnc2000);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Crossnet Co. <info@crossnet.co.jp>");
MODULE_DESCRIPTION("MTD map driver for Photron PNC-2000 board");
