/*
 *
 * Copyright (C) 2000 Coventive Co. <coventive@coventive.com>
 *
 * This code is GPL
 *
 */

#include <linux/module.h>
#include <linux/types.h>
#include <linux/kernel.h>

#include <linux/mtd/mtd.h>
#include <linux/mtd/map.h>
#include <linux/mtd/partitions.h>
#include <linux/delay.h> 
#include <asm/vr41xx.h>

//#define WINDOW_ADDR 0xbe800000
//#define WINDOW_ADDR 0xbf000000
//#define WINDOW_SIZE 0x00400000
//#define WINDOW_SIZE (0x00800000*2)

#if (CONFIG_PHL4000 || CONFIG_PHL4100)
#define WINDOW_ADDR 0xbe000000
#else
#define WINDOW_ADDR 0xbf000000
#endif


#if (CONFIG_PHL4000 || CONFIG_PHL4100)
#define WINDOW_SIZE (0x00800000*4)
#else
#define WINDOW_SIZE (0x00800000*2)
#endif


/* 
 * MAP DRIVER STUFF
 */

__u8 road_read8(struct map_info *map, unsigned long ofs)
{
  return *(__u8 *)(WINDOW_ADDR + ofs);
}

__u16 road_read16(struct map_info *map, unsigned long ofs)
{
  return *(__u16 *)(WINDOW_ADDR + ofs);
}

__u32 road_read32(struct map_info *map, unsigned long ofs)
{
  return *(volatile unsigned int *)(WINDOW_ADDR + ofs);
}

void road_copy_from(struct map_info *map, void *to, unsigned long from, ssize_t len)
{
  memcpy(to, (void *)(WINDOW_ADDR + from), len);
}

void road_write8(struct map_info *map, __u8 d, unsigned long adr)
{
  *(__u8 *)(WINDOW_ADDR + adr) = d;
}

void road_write16(struct map_info *map, __u16 d, unsigned long adr)
{
  *(__u16 *)(WINDOW_ADDR + adr) = d;
}

void road_write32(struct map_info *map, __u32 d, unsigned long adr)
{
  *(__u32 *)(WINDOW_ADDR + adr) = d;
}

void road_copy_to(struct map_info *map, unsigned long to, const void *from, ssize_t len)
{
  memcpy((void *)(WINDOW_ADDR + to), from, len);
}

void road_set_vpp(struct map_info *map, int Enable)
{

	if(Enable)
         *VR41XX_GIUPIODL  |= VR41XX_GIUPIODL_GPIO11;
	else
         *VR41XX_GIUPIODL  &= ~VR41XX_GIUPIODL_GPIO11;
}

struct map_info road_map = {
	name: "ROADHT",
	size: WINDOW_SIZE,
	buswidth: 2,
	read8: road_read8,
	read16: road_read16,
	read32: road_read32,
	copy_from: road_copy_from,
	write8: road_write8,
	write16: road_write16,
	write32: road_write32,
	copy_to: road_copy_to,
	set_vpp: road_set_vpp	 
			 
};


/*
 * MTD 'PARTITIONING' STUFF 
 */
static struct mtd_partition road_partitions[] = {
#if (CONFIG_PHL4000 || CONFIG_PHL4100)
        {
		 name: "/root",
		 size: 0x00100000, /* 1M */
		 offset: 0x01000000
	},
	{
	         name: "/",
	         size: 0x00100000, /* 1M */
	         offset: 0x01100000
	},
	{
	         name: "usr",
	         size: 0x00a00000, /* 10M */
	         offset: 0x01200000
	},
#if 0
        {
                 name: "bootloader",
                 size: 0x00100000, /* 1M */
                 offset: 0x01c00000
        },
#endif
        {
		 name: "kernel",
		 size: 0x00300000, /* 3M */
		 offset: 0x01d00000
	},
	{
	         name: "back-loader",
	         size: 0x00100000, /* 1M */
	         offset: 0x00000000
	},
	{
	         name: "USRETC",
                 size: 0x00300000, /* 3M */
                 offset: 0x00100000
        },
	{
	         name: "APP",
	         size: 0x00400000, /* 4M */
	         offset: 0x00400000
	},
	{
	         name: "USRFILE",
	         size: 0x00800000, /* 8M */
	         offset: 0x00800000
	}
	
#else	
	
        {
	
 		name: "backup_kernel",
   		size: 0x00200000,/*2M*/
   		//offset: 0x00400000
   		offset: 0x00000000
	},
	{
     		name: "usr", 
     		size: 0x00a00000,/*10M*/
     		offset: 0x00200000
	},
        {
	        name: "bootloader",
	        size: 0x00100000,/*1M*/
	     offset: 0x00c00000
	},
        {
     		name: "kernel",
     		size: 0x00200000,/*2M*/
     		offset: 0x00d00000
	},
        {
	        name: "ramdisk",
	        size: 0x00100000,/*1M*/
	        offset: 0x00f00000
	}

#endif
   
};

/* 
 * This is the master MTD device for which all the others are just
 * auto-relocating aliases.
 */
static struct mtd_info *mymtd;

#if LINUX_VERSION_CODE < 0x20212 && defined(MODULE)
#define init_road_mtd_map init_module
#define cleanup_road_mtd_map cleanup_module
#endif

int __init init_road_mtd_map(void)
{
   
	printk(KERN_NOTICE "ROADHT flash mapping: %x at %x\n", WINDOW_SIZE, WINDOW_ADDR);

        *VR41XX_GIUIOSELL |= VR41XX_GIUPIODL_GPIO11;
	*VR41XX_GIUPIODL &= ~VR41XX_GIUPIODL_GPIO11;
#if 0
	{
	int i;	
        for(i=0;i<16;i++)
             printk("%04x ", *(__u16 *)( WINDOW_ADDR + i));
        printk("\n");
        for(i=0;i<16;i++)
             printk("%04x ", *(__u16 *)( WINDOW_ADDR + 0x00100000 + i));
	}
#endif
       /* Enable flash read/write cycle */
        *(__u16 *)( VR41XX_BCUCNTREG1 )   |= 0x0050;
      
	mymtd = do_map_probe("cfi_probe", &road_map);
	if (mymtd) {
		int size;
		mymtd->module = THIS_MODULE;
		size = sizeof(road_partitions) / sizeof(struct mtd_partition);
		return add_mtd_partitions(mymtd, road_partitions, size);
	}
	return -ENXIO;
}

static void __exit cleanup_road_mtd_map(void)
{
 	if (mymtd) {
		del_mtd_partitions(mymtd);
		map_destroy(mymtd);
	}
}

module_init(init_road_mtd_map);
module_exit(cleanup_road_mtd_map);
