/*************************************************************
 *  s1d13706 control lcd Device
 *
 *  linux/drivers/video/s1d13706_control.c 
 *
 * Initial Version by: sam liao (sam_liao@coventive.com)
 *
 * This driver provides basic control for the lcd contrast and on_off
 * provided with road board.
 *
 * History:
 *      8/22/2001 - sam liao <sam_liao@coventive.com)
 *                  Initial Version. 
 * 
 *************************************************************/

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/init.h>
#include <linux/delay.h>
#include <asm/uaccess.h>
#include "Mr2ExtLcdcIO.h"

#define Contrast_VERSION	"2003-03-11 V1.1"
#define S1D13706_CONTROL_MAJOR 129
#define S1D13706_CONTROL_NAME "Lcd_Control"

#define MAX_CONTRAST   31
#define MIN_CONTRAST   0

#define S1D13706_POWER_IOCTL_ON      0
#define S1D13706_POWER_IOCTL_OFF     1
#define S1D13706_CONTRAST_IOCTL_UP   2
#define S1D13706_CONTRAST_IOCTL_DOWN 3
#define S1D13706_CONTRAST_IOCTL_GET  4
#define S1D13706_CONTRAST_IOCTL_SCALE 5

/* #define DEBUG */ /* Jeff masked 20020910 */

#ifdef DEBUG

#define S1D13706_CONTRAST_IOCTL_TEST 6
typedef struct PINOBJ
{
  char PinNo;
  char PinStatus;
}SETPIN;
static void set_s1d13760_GPIO(SETPIN *setpin);
#endif

#if 1 // def CONFIG_THERMOMETER /* [== added by tommy Mar.11.2003 === */
#define S1D3706_CONTRAST_STD_VAL_IOCTL_SET    7
#define S1D3706_CONTRAST_STD_VAL_IOCTL_GET    8
#define S1D3706_CONTRAST_ADJ_VAL_IOCTL_SET    9
#define S1D3706_CONTRAST_ADJ_VAL_IOCTL_GET    10
#define S1D3706_CONTRAST_AUTOSET_ENABLE       11
#define S1D3706_CONTRAST_AUTOSET_DISABLE      12
#endif                    /* === added by tommy Mar.11.2003 ==] */

//------------------------------------------------------------
void hot_key_contrast(int KeyValue);
static void extlcdc_76reg(void);
static void extlcdc_76ram(void);
static void contrast_set_init(void);
static void contrast_down(void);
static void contrast_up(void);
static void lcd_screen_off(void);
static void lcd_screen_on(void);
static void set_contrast_level(int *scale);

static int s1d13706_control_ioctl(struct inode * inode, struct file *filp,
			   unsigned int cmd , unsigned long arg);

static int s1d13706_control_open(struct inode *inode, struct file *filp);
static int s1d13706_control_close(struct inode *inode, struct file *filp);
//------------------------------------------------------------
static struct file_operations s1d13706_control_fops;

#if 1 //def CONFIG_THERMOMETER  /* [== modified by tommy Mar.11.2003 === */
/* for contrast auto-set function */
#define MAX_CON_ADJ_VAL   6
#define MIN_CON_ADJ_VAL  -6
#define MAX_CON_STD_VAL  31
#define MIN_CON_STD_VAL   0
int iConStdVal = 0;  /* contrast standard value    0 ~ 31  */   
int iConAdjVal = 0;   /* contrast adjustment value  -6 ~ +6 */
int bAutoAdjCont = 1;
static int ContrastValue = 0;
#else 
static int ContrastValue = 15;
#endif                     /* === modified by tommy Mar.11.2003 ==] */

//EXPORT_SYMBOL(hot_key_contrast);

//---------------------------------------------------------------------
//#ifdef CONFIG_PM_SUSPEND_WAKEUP
void pm_contrast_resume(void)
{
   
   int i;
   int resumeval;

#if 1 // def CONFIG_THERMOMETER /* [== added by tommy Mar.11.2003 === */
   if ( iConStdVal + iConAdjVal > MAX_CONTRAST )      ContrastValue = MAX_CONTRAST;
   else if ( iConStdVal + iConAdjVal < MIN_CONTRAST ) ContrastValue = MIN_CONTRAST;
   else                             ContrastValue = iConStdVal + iConAdjVal;  
#endif                    /* === added by tommy Mar.11.2003 ==] */
   
   resumeval = ContrastValue;
   
   ContrastValue = 31;	
   
   while ( 1 ) {
      if ( LCD_Contrast_Down() == -1 )
	break;      
   }	
   
   for ( i =0; i <= resumeval; i++ )
     LCD_Contrast_Up();
   
}
//#endif

void hot_key_contrast(int up)
{
#if 1 // def CONFIG_THERMOMETER /* [== modified by tommy Mar.11.2003 === */
  int iSetVal;
  
  
  if ( up )
  {
  	if ( iConAdjVal < MAX_CON_ADJ_VAL )  iConAdjVal++;		
	if ( iConStdVal + iConAdjVal > MAX_CONTRAST )  iSetVal = MAX_CONTRAST;
	else                                 iSetVal = iConStdVal + iConAdjVal;	
  	if ( bAutoAdjCont == 0 ) 
  	{
  		/* printk("(Disable Up) Std=%d, Adj=%d, now=%d, Set=%d\n",
			iConStdVal, iConAdjVal, ContrastValue, iSetVal );*/
	  return;
  	}
	if ( iSetVal != ContrastValue )
	{
	set_contrast_level(&iSetVal);
//	ContrastValue = iSetVal;
	}
        /* printk("(up) Std=%d, Adj=%d, now =%d\n", iConStdVal, iConAdjVal, ContrastValue ); */
  }
  else
  {
	if ( iConAdjVal > MIN_CON_ADJ_VAL ) iConAdjVal--;
        if ( iConStdVal + iConAdjVal < MIN_CONTRAST )  iSetVal = MIN_CONTRAST;
        else                                iSetVal = iConStdVal + iConAdjVal;
  	if ( bAutoAdjCont == 0 ) 
  	{
  		/*printk("(Disable Down) Std=%d, Adj=%d, now=%d, Set=%d\n",
			iConStdVal, iConAdjVal, ContrastValue, iSetVal ); */
	  return;
  	}
	if ( iSetVal != ContrastValue )
	{
        set_contrast_level(&iSetVal);
//      ContrastValue = iSetVal;		
	}
        /* printk("(down) Std=%d, Adj=%d, now=%d\n", iConStdVal, iConAdjVal, ContrastValue );  */
  }	
#else 			     
   int setval;
      
  if( (up) && (ContrastValue < 31) )
  {
     setval = ContrastValue;
     setval++;
     set_contrast_level(&setval);

     //contrast_up();
     //ContrastValue++;
     
  }
  if( (!up) && (ContrastValue > 0)  )
  {
     setval = ContrastValue;
     setval--;
     set_contrast_level(&setval);
   //contrast_down();
   //ContrastValue--;
  }
#endif                    /* === modified by tommy Mar.11.2003 ==] */  
}

#if 1 // def CONFIG_THERMOMETER /* [== added by tommy Mar.11.2003 === */
int AutoSetContrastValue( int iStdVal )
{
  int iSetVal;

  
  if ( iStdVal > MAX_CON_STD_VAL ) iStdVal = MAX_CON_STD_VAL;
  if ( iStdVal < MIN_CON_STD_VAL ) iStdVal = MIN_CON_STD_VAL;

//  if ( iConAdjVal > MAX_CON_ADJ_VAL ) iConAdjVal = MAX_CON_ADJ_VAL;
//  if ( iConAdjVal < MIN_CON_ADJ_VAL ) iConAdjVal = MIN_CON_ADJ_VAL; 
  
  iConStdVal = iStdVal;  
  iSetVal = iConStdVal + iConAdjVal;
  if ( iSetVal > MAX_CONTRAST ) 
  {
	  iSetVal = MAX_CONTRAST;
	  iConAdjVal = iSetVal - iConStdVal;
  }
  
  if ( iSetVal < MIN_CONTRAST ) 
  {
	  iSetVal = MIN_CONTRAST; 
	  iConAdjVal = MIN_CONTRAST - iConStdVal; 
  }
  
  if ( bAutoAdjCont == 0 ) 
  {
	  /* printk("(Disable SET) Std=%d, Adj=%d, now=%d, Set=%d",
                    iConStdVal, iConAdjVal, ContrastValue, iSetVal ); */
	  return 0;
  }
  
  /* printk("(SET) Std=%d, Adj=%d, now=%d, Set=%d", 
		  iConStdVal, iConAdjVal, ContrastValue, iSetVal ); */
  
  if ( iSetVal != ContrastValue )
  	set_contrast_level(&iSetVal);
  
//ContrastValue = iSetVal;   
  return 0;
}

int SetAdjustContrastValue( int iAdjVal )
{
  int iSetVal;


  if ( iAdjVal > MAX_CON_ADJ_VAL ) iAdjVal = MAX_CON_ADJ_VAL;
  if ( iAdjVal < MIN_CON_ADJ_VAL ) iAdjVal = MIN_CON_ADJ_VAL; 
  
//  if ( iConStdVal > MAX_CON_STD_VAL ) iConStdVal = MAX_CON_STD_VAL;
//  if ( iConStdVal < MIN_CON_STD_VAL ) iConStdVal = MIN_CON_STD_VAL;
  
  iConAdjVal = iAdjVal;  
  
  iSetVal = iConStdVal + iConAdjVal;
  if ( iSetVal > MAX_CONTRAST ) 
  {
	  iSetVal = MAX_CONTRAST;
  }
  
  if ( iSetVal < MIN_CONTRAST ) 
  {
	  iSetVal = MIN_CONTRAST; 
  }

  if ( bAutoAdjCont == 0 ) 
  {
	  /* printk("(Disable) Std=%d, Adj=%d, now=%d, Set=%d",
                    iConStdVal, iConAdjVal, ContrastValue, iSetVal ); */
	  return 0;
  }
  /* printk("(SET) Std=%d, Adj=%d, now=%d, Set=%d",  
		  iConStdVal, iConAdjVal, ContrastValue, iSetVal ); */
  
  if ( iSetVal != ContrastValue )
  	set_contrast_level(&iSetVal);
  
//ContrastValue = iSetVal;   
  return 0;
}
#endif                    /* === added by tommy Mar.11.2003 ==] */

static void extlcdc_76reg(void)
{
        // MR-2A GPIO3
        // GPIO3/CSOUT3 LCDC_MOD Command/Memory Select Signal
        // When the signal is low, the sed1376 registers could be accessed.
        if( ( *( ( volatile unsigned short *)MR2GIUPIOD ) & 0x0008 ) == 0x0008 )
        {
                *( ( volatile unsigned short *)MR2GIUDIR ) |= 0x0008;
                *( ( volatile unsigned short *)MR2GIUPIOD ) ^= 0x0008;
        }
}

static void extlcdc_76ram(void)
{
        // MR-2A GPIO3
        // GPIO3/CSOUT3 LCDC_MOD Command/Memory Select Signal
        // When the signal is high, the sed1376 memory could be accessed.
        if( ( *( ( volatile unsigned short *)MR2GIUPIOD ) & 0x0008 ) != 0x0008 )
        {
                *( ( volatile unsigned short *)MR2GIUDIR ) |= 0x0008;
                *( ( volatile unsigned short *)MR2GIUPIOD ) |= 0x0008;
        }
}


#ifdef DEBUG
static void set_s1d13760_GPIO(SETPIN *setpin)
{
  char pinNo;
  char status;
  char pin = 0x01;
  pinNo = setpin->PinNo;
  status = setpin->PinStatus;

  extlcdc_76reg();
  udelay(10);
  pin = pin << pinNo;

if( status == 1)
  *((volatile unsigned char *)SED1376REG_AC) |= pin; 
else
  *((volatile unsigned char *)SED1376REG_AC) &= (!pin);

  extlcdc_76ram();
}
#endif

#if 1 /* tommy bootloader */
static int LCD_Contrast_Up(void)
{
   if ( ContrastValue > 31 ) return -1;
   
   extlcdc_76reg();

   *( ( unsigned char * )SED1376REG_AC ) |= 0x04;	
   udelay(8);
   *( ( unsigned char * )SED1376REG_AC ) |= 0x02;
   udelay(8);
   *( ( unsigned char * )SED1376REG_AC ) &= 0xFD;  
   udelay(8);

   extlcdc_76ram();                                                                                /* SED1376~BLVRAM~CA~CN~CZ~CX ~AK M/R#~R[~Nq~Bp"HIGH"~BI~P]~Rh */

   ContrastValue++;
   
   return 0;
}	
	
static int LCD_Contrast_Down(void)
{
  if ( ContrastValue < 0 )  return -1;
	
   extlcdc_76reg();

   *( ( unsigned char * )SED1376REG_AC ) &= 0xFB;    
   udelay(8);
   *( ( unsigned char * )SED1376REG_AC ) |= 0x02;
   udelay(8);
   *( ( unsigned char * )SED1376REG_AC ) &= 0xFD;  
   udelay(8);
 
   extlcdc_76ram();                                                                                /* SED1376~BLVRAM~CA~CN~CZ~CX ~AK M/R#~R[~Nq~Bp"HIGH"~BI~P]~Rh */
   
   ContrastValue--;
   
   return 0;
}

static void set_contrast_level(int *scale)
{
   int i,range;
   int level = *scale;
     
   if ( level < 0 || level > 31 ) return;  
   if ( level == ContrastValue )  return;


   *( ( unsigned char * )SED1376REG_AC ) &= ~0x01; /* CS = LOW */
   if ( level > ContrastValue ) {
      range = level - ContrastValue;
      for ( i = 0; i < range; i++ )
	LCD_Contrast_Up();	    
    } else {
       range = ContrastValue - level;
       for ( i = 0; i < range; i++ )
       LCD_Contrast_Down();	    
    }	   
   *( ( unsigned char * )SED1376REG_AC ) |= 0x01; /* CS = HIGH */
}	

#endif

static void contrast_set_init(void)
{
  /*GPIO0= hi ,GPIO1= lo, GPIO2= lo; */ 
  *((volatile unsigned char *)SED1376REG_AC) |= 0x01;/*0000 0001 gpio0= H*/
  *((volatile unsigned char *)SED1376REG_AC) &= 0xfd;/*1111 1101 gpio1= L*/
  *((volatile unsigned char *)SED1376REG_AC) &= 0xfb;/*1111 1011 gpio2= L*/
}

static void contrast_up(void)
{
 if(ContrastValue >= 31 )
 {
	 //printk("Contrast large than 31\n");
	 return;
 }
 
 extlcdc_76reg();
 contrast_set_init();
  *((volatile unsigned char *)SED1376REG_AC) |= 0x04;/*0000 0100 gpio2= H*/
  *((volatile unsigned char *)SED1376REG_AC) &= 0xfe;/*1111 1110 gpio0= L*/
  *((volatile unsigned char *)SED1376REG_AC) |= 0x02;/*0000 0010 gpio1= H*/
  ContrastValue++;
  udelay(5);
  *((volatile unsigned char *)SED1376REG_AC) &= 0xfd;/*1111 1101 gpio1= L*/
  udelay(5);
  *((volatile unsigned char *)SED1376REG_AC) |= 0x01;/*0000 0010 gpio0= H*/
    
 contrast_set_init();
 extlcdc_76ram();
	
}

static void contrast_down(void)
{
 //if(ContrastValue < 0 )return;
 if(ContrastValue <= 0 )
 {
	 //printk("Contrast small than 0\n");
	 return;
 }
 
 extlcdc_76reg();
 contrast_set_init();
  *((volatile unsigned char *)SED1376REG_AC) &= 0xfb;/*1111 1011 gpio2= L*/
  *((volatile unsigned char *)SED1376REG_AC) &= 0xfe;/*1111 1110 gpio0= L*/
  *((volatile unsigned char *)SED1376REG_AC) |= 0x02;/*0000 0010 gpio1= H*/
  ContrastValue--;
  udelay(5);
  *((volatile unsigned char *)SED1376REG_AC) &= 0xfd;/*1111 1101 gpio1= L*/
  udelay(5);
  *((volatile unsigned char *)SED1376REG_AC) |= 0x01;/*0000 0010 gpio0= H*/
    
 contrast_set_init();
 extlcdc_76ram();
}
#if 0
static void set_contrast_level(int *scale)
{
   int UpDownStep,i;
   int UpFlag;
   
   if( ContrastValue == *scale ) return;

   if( *scale > 63 || *scale < 0 ) return;
   
   //printk("Contrast value = %d, scale = %d\n",ContrastValue, *scale);
   if(  ContrastValue < *scale ) {
      UpFlag = 1; 
      UpDownStep = (*scale) - ContrastValue;
   } else {
      UpFlag = 0;
      UpDownStep = ContrastValue - (*scale);
   }

   extlcdc_76reg();
   contrast_set_init();
   
   if(UpFlag == 1)
     *((volatile unsigned char *)SED1376REG_AC) |= 0x04;/*0000 0100 gpio2= H*/
   else
     *((volatile unsigned char *)SED1376REG_AC) &= 0xfb;/*1111 1011 gpio2= L*/
   *((volatile unsigned char *)SED1376REG_AC) &= 0xfe;/*1111 1110 gpio0= L*/

   while(1)  {
      if(UpDownStep == 0 ) break;
      
      udelay(5);
      *((volatile unsigned char *)SED1376REG_AC) |= 0x02;/*0000 0010 gpio1= H*/
      udelay(5);
      *((volatile unsigned char *)SED1376REG_AC) &= 0xfd;/*1111 1101 gpio1= L*/
      UpDownStep--;
      
      if(UpFlag == 1) 
	ContrastValue++;
      else 
	ContrastValue--;
      
      //if( ContrastValue > 31 || ContrastValue < 0 ) break;	
   }
   *((volatile unsigned char *)SED1376REG_AC) |= 0x01;/*0000 0010 gpio0= H*/
   
   contrast_set_init();
   extlcdc_76ram();
}
#endif
void lcd_screen_on(void)
{

        // GPIO6 set to be output 
        *(unsigned short *)MR2GIUDIR |= 0x0040;

        // GPIO6 set to high, blacklight on.
        *(unsigned short *)MR2GIUPIOD |= 0x0040;
}

void lcd_screen_off(void)
{
        // GPIO6 set to be output
        *(unsigned short *)MR2GIUDIR |= 0x0040;

        // GPIO6 set to low, blacklight off.
        *(unsigned short *)MR2GIUPIOD &= 0xffbf;
}

//---------------------------------------------------------------------
static int s1d13706_control_open(struct inode *inode, struct file *filp)
{
    MOD_INC_USE_COUNT;
    return 0;
}

static int s1d13706_control_close(struct inode *inode, struct file *filp)
{
	MOD_DEC_USE_COUNT;
    return 0;
}

static int s1d13706_control_ioctl(struct inode * inode, struct file *filp,
				   unsigned int cmd , unsigned long arg)
{
	int scale;
	int ret;
#ifdef DEBUG
        SETPIN setpin;
#endif
	ret = (-EINVAL);
	
	switch(cmd) 
	  {
          case S1D13706_POWER_IOCTL_ON:
		lcd_screen_on();
	    break;
	  case S1D13706_POWER_IOCTL_OFF:
		lcd_screen_off();
	    break;
	  case S1D13706_CONTRAST_IOCTL_UP:
		//contrast_up();
		LCD_Contrast_Up();
	    break;
	  case S1D13706_CONTRAST_IOCTL_DOWN:
		//contrast_down();
		LCD_Contrast_Down();
		break;
	  case S1D13706_CONTRAST_IOCTL_GET:
               return copy_to_user((void *) arg, &ContrastValue, 
                                             sizeof(ContrastValue)) ? -EFAULT :0 ;
		break;
	  case S1D13706_CONTRAST_IOCTL_SCALE:
		if (copy_from_user(&scale, (void *)arg, sizeof(scale)))
                return - EFAULT;
            	set_contrast_level(&scale);
		break;
	
#ifdef DEBUG
	  case S1D13706_CONTRAST_IOCTL_TEST:
          if (copy_from_user(&setpin, (void *)arg, sizeof(SETPIN)))
              return - EFAULT;
	  set_s1d13760_GPIO(&setpin);
		break;
#endif

#if 1 // def CONFIG_THERMOMETER  /* [== added by tommy Mar.11.2003 === */
	  case S1D3706_CONTRAST_STD_VAL_IOCTL_SET:
	       if ( copy_from_user(&scale, (void*)arg, sizeof(scale)))
                 return -EFAULT;
	       AutoSetContrastValue( scale );	       
	       break;		
	       
	  case S1D3706_CONTRAST_STD_VAL_IOCTL_GET:
               if ( copy_to_user((void *) arg, &iConStdVal, sizeof(iConStdVal)))
		 return -EFAULT;
               break;
	       
	  case S1D3706_CONTRAST_ADJ_VAL_IOCTL_SET:
	       if ( copy_from_user(&scale, (void*)arg, sizeof(scale)))
                 return -EFAULT;
               SetAdjustContrastValue( scale );
	       break;
	  	
	  case S1D3706_CONTRAST_ADJ_VAL_IOCTL_GET:
               return copy_to_user((void *) arg, &iConAdjVal, 
			       sizeof(iConAdjVal)) ? -EFAULT :0;
               break;

	  case S1D3706_CONTRAST_AUTOSET_DISABLE:
	       /* printk("=====contrast auto-set disable=====\n"); */
	       bAutoAdjCont = 0;
               break;
	       
	  case S1D3706_CONTRAST_AUTOSET_ENABLE:     
	       /* printk("=====contrast auto-set enable=====\n"); */
	       bAutoAdjCont = 1;
	       break;
#endif                     /* === added by tommy Mar.11.2003 ==] */

	  default:
	    break;
	  }
	
	//return ret;
	return 0;	/* For normal end 2003-0714 */
}

//------------------------------------------------------------
int __init s1d13706_control_init(void)
{
  int result;
  int s1d13706_control_major;
  s1d13706_control_major = S1D13706_CONTROL_MAJOR;

  result = register_chrdev(S1D13706_CONTROL_MAJOR,S1D13706_CONTROL_NAME,
						 &s1d13706_control_fops);
  if (result < 0) {
    /* printk(KERN_WARNING "1D13706 CONTROL MAIOR: cant get major %d\n", 
	  					 s1d13706_control_major); */
    return result;
  }
  //printk("s1d13706_control Driver Initialized...\n");
  printk(KERN_INFO "Contrast Driver v" Contrast_VERSION "\n");
  return 0;

}


#ifdef MODULE
module_init(sa1100frtlight_init);
#endif

//------------------------------------------------------------

#ifdef MODULE
void __exit sa1100frtlight_exit(void)
{
  unregister_chrdev(sa1100_frontlight_major, "sa1100-fl");
  printk("s1d13706_control Driver Unloaded\n");
}
module_exit(sa1100frtlight_exit);
#endif

//------------------------------------------------------------
static struct file_operations s1d13706_control_fops = {
        ioctl:          s1d13706_control_ioctl,
        open:           s1d13706_control_open,
        release:        s1d13706_control_close,
};

//-------------------------------------------------------------
