/*
 *  EPSON SED1376F0A frame buffer device
 *  ALPS LRH6W503XA mono 160x240x4
 *
 *  Only for mono 160x240 4 bpp
 *
 *  Copyright (C) 2001-6-1 CIH <cih@coventive.com>
 *
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/tty.h>
#include <linux/slab.h>
#include <linux/vmalloc.h>
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <asm/uaccess.h>
#include <linux/fb.h>
#include <linux/init.h>
#include <linux/timer.h>

#include <video/fbcon.h>
#include <video/fbcon-cfb4.h>

#include "Mr2ExtLcdcIO.h"
#include "Mr2ExtLcdcVar.h"
#include "Mr2ExtLcdcTabl.h"

#define FB_VERSION	"2003-03-10 V1.0"
#define VIDEOMEMSIZE	(160*240*4/8)

#define CIH

static u_long videomemory, videomemorysize = VIDEOMEMSIZE;
static int currcon = 0;
static struct display disp;
static struct fb_info sed1376fb_info;
static char sed1376fb_name[16] = "SED1376 FB";
static int cur_level = -1;
static struct fb_var_screeninfo sed1376fb_default =
{
    /* 160x240, 4 bpp */
    160, 240, 160, 240, 0, 0, 4, 1,
    {0, 4, 0}, {0, 4, 0}, {0, 4, 0}, {0, 0, 0},
    0, 0, -1, -1, 0, 20000, 0, 0, 0, 0, 0, 0,
    0, FB_VMODE_NONINTERLACED
};

int sed1376fb_setup(char*);

static int sed1376fb_get_fix(struct fb_fix_screeninfo *fix, int con,
		       struct fb_info *info);
static int sed1376fb_get_var(struct fb_var_screeninfo *var, int con,
		       struct fb_info *info);
static int sed1376fb_set_var(struct fb_var_screeninfo *var, int con,
		       struct fb_info *info);
static int sed1376fb_pan_display(struct fb_var_screeninfo *var, int con,
			   struct fb_info *info);
static int sed1376fb_get_cmap(struct fb_cmap *cmap, int kspc, int con,
			struct fb_info *info);
static int sed1376fb_set_cmap(struct fb_cmap *cmap, int kspc, int con,
			struct fb_info *info);

int sed1376fb_init(void);
static int sed1376fbcon_switch(int con, struct fb_info *info);
static int sed1376fbcon_updatevar(int con, struct fb_info *info);
static void sed1376fbcon_blank(int blank, struct fb_info *info);

static u_long get_line_length(int xres_virtual, int bpp);
static void sed1376fb_encode_fix(struct fb_fix_screeninfo *fix,
			   struct fb_var_screeninfo *var);
static void set_color_bitfields(struct fb_var_screeninfo *var);
static int sed1376fb_getcolreg(u_int regno, u_int *red, u_int *green, u_int *blue,
			 u_int *transp, struct fb_info *info);
static int sed1376fb_setcolreg(u_int regno, u_int red, u_int green, u_int blue,
			 u_int transp, struct fb_info *info);
static void do_install_cmap(int con, struct fb_info *info);

static void extlcdc_76reg(void);
static void extlcdc_76ram(void);
static u_long set_sed1376_mono_160x240x4(void);
static void contrast(int level);
static void contrast_up(void);
static void contrast_down(void);
#ifdef CIH
static void contrast_default(void);
#else
static void contrast_set_init(count Up_Down);
static void INC_plus_GPIO1(int plus);
#endif
static void blank_screen(void);
#if 0
static struct timer_list lcd_timer;
#endif
static int blankinterval = 30000*HZ; // 30000 sec
static int blank = 0;
static int Mask=0;
#if 0
int pm_access_device(void);
#endif
void lcd_screen_on(void);
void lcd_screen_off(void);
void set_blank_interval(int interval);
void set_pow_low_interval(int interval);
int read_lcd_off_interval(void);
void set_blank_mask(int mask);
int read_blank_mask(void);

//EXPORT_SYMBOL(lcd_screen_on);

static struct fb_ops sed1376fb_ops =
{
	owner:		THIS_MODULE,
	fb_get_fix:	sed1376fb_get_fix,
	fb_get_var:	sed1376fb_get_var,
	fb_set_var:	sed1376fb_set_var,
	fb_get_cmap:	sed1376fb_get_cmap,
	fb_set_cmap:	sed1376fb_set_cmap,
	fb_pan_display: sed1376fb_pan_display,
};

static int sed1376fb_get_fix(struct fb_fix_screeninfo *fix, int con,
		       struct fb_info *info)
{
    struct fb_var_screeninfo *var;

    if (con == -1)
	var = &sed1376fb_default;
    else
	var = &fb_display[con].var;

    sed1376fb_encode_fix(fix, var);

    return 0;
}

static int sed1376fb_get_var(struct fb_var_screeninfo *var, int con,
		       struct fb_info *info)
{
    if (con == -1)
	*var = sed1376fb_default;
    else
	*var = fb_display[con].var;

    set_color_bitfields(var);

    return 0;
}

static int sed1376fb_set_var(struct fb_var_screeninfo *var, int con,
		       struct fb_info *info)
{
    int err, activate = var->activate;
    int oldxres, oldyres, oldvxres, oldvyres, oldbpp;
    u_long line_length;
    struct display *display;

    if (con >= 0)
	display = &fb_display[con];
    else
	display = &disp;

    if (var->vmode & FB_VMODE_CONUPDATE)
    {
	var->vmode |= FB_VMODE_YWRAP;
	var->xoffset = display->var.xoffset;
	var->yoffset = display->var.yoffset;
    }

    if (!var->xres)
	var->xres = 1;

    if (!var->yres)
	var->yres = 1;

    if (var->xres > var->xres_virtual)
	var->xres_virtual = var->xres;

    if (var->yres > var->yres_virtual)
	var->yres_virtual = var->yres;

    var->bits_per_pixel = 4;

    line_length = get_line_length(var->xres_virtual, var->bits_per_pixel);

    if (line_length*var->yres_virtual > videomemorysize)
	return -ENOMEM;

    set_color_bitfields(var);

    if ((activate & FB_ACTIVATE_MASK) == FB_ACTIVATE_NOW)
    {
	oldxres = display->var.xres;
	oldyres = display->var.yres;
	oldvxres = display->var.xres_virtual;
	oldvyres = display->var.yres_virtual;
	oldbpp = display->var.bits_per_pixel;
	display->var = *var;

	if (oldxres != var->xres || oldyres != var->yres ||
	    oldvxres != var->xres_virtual || oldvyres != var->yres_virtual ||
	    oldbpp != var->bits_per_pixel)
	{
	    struct fb_fix_screeninfo fix;

	    sed1376fb_encode_fix(&fix, var);
	    display->screen_base = (char *)videomemory;
	    display->visual = fix.visual;
	    display->type = fix.type;
	    display->type_aux = fix.type_aux;
	    display->ypanstep = fix.ypanstep;
	    display->ywrapstep = fix.ywrapstep;
	    display->line_length = fix.line_length;
	    display->can_soft_blank = 1;
	    display->inverse = 0;

	    display->dispsw = &fbcon_cfb4;

	    if (sed1376fb_info.changevar)
		(*sed1376fb_info.changevar)(con);
	}

	if (oldbpp != var->bits_per_pixel)
	{
	    if ((err = fb_alloc_cmap(&display->cmap, 0, 0)))
		return err;

	    do_install_cmap(con, info);
	}
    }

    return 0;
}

static int sed1376fb_pan_display(struct fb_var_screeninfo *var, int con,
			   struct fb_info *info)
{
    if (var->vmode & FB_VMODE_YWRAP)
    {
	if (var->yoffset < 0 ||
	    var->yoffset >= fb_display[con].var.yres_virtual ||
	    var->xoffset)
	    return -EINVAL;
    }
    else
    {
	if (var->xoffset+fb_display[con].var.xres >
	    fb_display[con].var.xres_virtual ||
	    var->yoffset+fb_display[con].var.yres >
	    fb_display[con].var.yres_virtual)
	    return -EINVAL;
    }

    fb_display[con].var.xoffset = var->xoffset;
    fb_display[con].var.yoffset = var->yoffset;

    if (var->vmode & FB_VMODE_YWRAP)
	fb_display[con].var.vmode |= FB_VMODE_YWRAP;
    else
	fb_display[con].var.vmode &= ~FB_VMODE_YWRAP;

    return 0;
}

static int sed1376fb_get_cmap(struct fb_cmap *cmap, int kspc, int con,
			struct fb_info *info)
{
    if (con == currcon)
	return fb_get_cmap(cmap, kspc, sed1376fb_getcolreg, info);
    else if (fb_display[con].cmap.len)
	fb_copy_cmap(&fb_display[con].cmap, cmap, kspc ? 0 : 2);
    else
	fb_copy_cmap(fb_default_cmap(1<<fb_display[con].var.bits_per_pixel),
		     cmap, kspc ? 0 : 2);

    return 0;
}

static int sed1376fb_set_cmap(struct fb_cmap *cmap, int kspc, int con,
			struct fb_info *info)
{
    int err;

    if (!fb_display[con].cmap.len)
    {
	if ((err = fb_alloc_cmap(&fb_display[con].cmap,
			      1<<fb_display[con].var.bits_per_pixel, 0)))
	    return err;
    }

    if (con == currcon)
	return fb_set_cmap(cmap, kspc, sed1376fb_setcolreg, info);
    else
	fb_copy_cmap(cmap, &fb_display[con].cmap, kspc ? 0 : 1);

    return 0;
}


int __init sed1376fb_setup(char *options)
{
    char *this_opt;

    sed1376fb_info.fontname[0] = '\0';

    if (!options || !*options)
	return 0;

    for (this_opt = strtok(options, ","); this_opt;
	 this_opt = strtok(NULL, ","))
    {
	if (!strncmp(this_opt, "font:", 5))
	    strcpy(sed1376fb_info.fontname, this_opt+5);
    }

    return 0;
}


int __init sed1376fb_init(void)
{
    videomemory = set_sed1376_mono_160x240x4();

    printk(KERN_INFO "FrameBuffer Driver v" FB_VERSION "\n");
    strcpy(sed1376fb_info.modename, sed1376fb_name);
    sed1376fb_info.changevar = NULL;
    sed1376fb_info.node = -1;
    sed1376fb_info.fbops = &sed1376fb_ops;
    sed1376fb_info.disp = &disp;
    sed1376fb_info.switch_con = &sed1376fbcon_switch;
    sed1376fb_info.updatevar = &sed1376fbcon_updatevar;
    sed1376fb_info.blank = &sed1376fbcon_blank;
    sed1376fb_info.flags = FBINFO_FLAG_DEFAULT;

    sed1376fb_set_var(&sed1376fb_default, -1, &sed1376fb_info);

    if (register_framebuffer(&sed1376fb_info) < 0)
	return -EINVAL;

    return 0;
}


static int sed1376fbcon_switch(int con, struct fb_info *info)
{
    if (fb_display[currcon].cmap.len)
	fb_get_cmap(&fb_display[currcon].cmap, 1, sed1376fb_getcolreg, info);

    currcon = con;

    do_install_cmap(con, info);

    return 0;
}

static int sed1376fbcon_updatevar(int con, struct fb_info *info)
{
    return 0;
}

static void sed1376fbcon_blank(int blank, struct fb_info *info)
{
}

static u_long get_line_length(int xres_virtual, int bpp)
{
    u_long length;

    length = xres_virtual*bpp;
    length = (length+31)&-32;
    length >>= 3;

    return(length);
}

static void sed1376fb_encode_fix(struct fb_fix_screeninfo *fix,
			   struct fb_var_screeninfo *var)
{
    memset(fix, 0, sizeof(struct fb_fix_screeninfo));
    strcpy(fix->id, sed1376fb_name);
    fix->smem_start = videomemory;
    fix->smem_len = videomemorysize;
    fix->type = FB_TYPE_PACKED_PIXELS;
    fix->type_aux = 0;

    fix->visual = FB_VISUAL_PSEUDOCOLOR;

    fix->ywrapstep = 1;
    fix->xpanstep = 1;
    fix->ypanstep = 1;
    fix->line_length = get_line_length(var->xres_virtual, var->bits_per_pixel);
}

static void set_color_bitfields(struct fb_var_screeninfo *var)
{
    var->red.msb_right = 0;
    var->green.msb_right = 0;
    var->blue.msb_right = 0;
    var->transp.msb_right = 0;
}

static int sed1376fb_getcolreg(u_int regno, u_int *red, u_int *green, u_int *blue,
			 u_int *transp, struct fb_info *info)
{
    return 1;
}

static int sed1376fb_setcolreg(u_int regno, u_int red, u_int green, u_int blue,
			 u_int transp, struct fb_info *info)
{
    return 1;
}


static void do_install_cmap(int con, struct fb_info *info)
{
    if (con != currcon)
	return;

    if (fb_display[con].cmap.len)
	fb_set_cmap(&fb_display[con].cmap, 1, sed1376fb_setcolreg, info);
    else
	fb_set_cmap(fb_default_cmap(1<<fb_display[con].var.bits_per_pixel), 1,
		    sed1376fb_setcolreg, info);
}

#ifdef MODULE
int init_module(void)
{
    return sed1376fb_init();
}

void cleanup_module(void)
{
    unregister_framebuffer(&sed1376fb_info);
}

#endif /* MODULE */

static void extlcdc_76reg(void)
{
	// MR-2A GPIO3
	// GPIO3/CSOUT3 LCDC_MOD Command/Memory Select Signal
	// When the signal is low, the sed1376 registers could be accessed.
	if( ( *( ( unsigned short *)MR2GIUPIOD ) & 0x0008 ) == 0x0008 )
	{
		*( ( unsigned short *)MR2GIUDIR ) |= 0x0008;
		*( ( unsigned short *)MR2GIUPIOD ) ^= 0x0008;
	}
}

static void extlcdc_76ram(void)
{
	// MR-2A GPIO3
	// GPIO3/CSOUT3 LCDC_MOD Command/Memory Select Signal
	// When the signal is high, the sed1376 memory could be accessed.
	if( ( *( ( unsigned short *)MR2GIUPIOD ) & 0x0008 ) != 0x0008 )
	{
		*( ( unsigned short *)MR2GIUDIR ) |= 0x0008;
		*( ( unsigned short *)MR2GIUPIOD ) |= 0x0008;
	}
}

static u_long set_sed1376_mono_160x240x4(void)
{
	//unsigned char mem_dat = 0;
	unsigned long i = 0;
	//unsigned long j = 0;

	// send high level to MR-2A GPIO7
	// LCDC_RST LCDC reset signal
	*(unsigned short *)MR2GIUDIR |= 0x0080;
	*(unsigned short *)MR2GIUPIOD |= 0x0080;

	extlcdc_76reg();

	*( ( unsigned char *)SED1376REG_04 ) = ini_76reg_04;
	*( ( unsigned char *)SED1376REG_05 ) = ini_76reg_05;
	*( ( unsigned char *)SED1376REG_10 ) = ini_76reg_10;
	*( ( unsigned char *)SED1376REG_11 ) = ini_76reg_11;
	*( ( unsigned char *)SED1376REG_12 ) = ini_76reg_12;
	*( ( unsigned char *)SED1376REG_14 ) = ini_76reg_14;
	*( ( unsigned char *)SED1376REG_16 ) = ini_76reg_16;
	*( ( unsigned char *)SED1376REG_17 ) = ini_76reg_17;
	*( ( unsigned char *)SED1376REG_18 ) = ini_76reg_18;
	*( ( unsigned char *)SED1376REG_19 ) = ini_76reg_19;
	*( ( unsigned char *)SED1376REG_1C ) = ini_76reg_1C;
	*( ( unsigned char *)SED1376REG_1D ) = ini_76reg_1D;
	*( ( unsigned char *)SED1376REG_1E ) = ini_76reg_1E;
	*( ( unsigned char *)SED1376REG_1F ) = ini_76reg_1F;
	*( ( unsigned char *)SED1376REG_20 ) = ini_76reg_20;
	*( ( unsigned char *)SED1376REG_22 ) = ini_76reg_22;
	*( ( unsigned char *)SED1376REG_23 ) = ini_76reg_23;
	*( ( unsigned char *)SED1376REG_24 ) = ini_76reg_24;
	*( ( unsigned char *)SED1376REG_26 ) = ini_76reg_26;
	*( ( unsigned char *)SED1376REG_27 ) = ini_76reg_27;
	*( ( unsigned char *)SED1376REG_70 ) = ini_76reg_70;
	*( ( unsigned char *)SED1376REG_71 ) = ini_76reg_71;
//	*( ( unsigned char *)SED1376REG_71 ) = 0x01;
	*( ( unsigned char *)SED1376REG_74 ) = ini_76reg_74;
	*( ( unsigned char *)SED1376REG_75 ) = ini_76reg_75;
	*( ( unsigned char *)SED1376REG_76 ) = ini_76reg_76;
	*( ( unsigned char *)SED1376REG_78 ) = ini_76reg_78;
	*( ( unsigned char *)SED1376REG_79 ) = ini_76reg_79;
	*( ( unsigned char *)SED1376REG_7C ) = ini_76reg_7C;
	*( ( unsigned char *)SED1376REG_7D ) = ini_76reg_7D;
	*( ( unsigned char *)SED1376REG_7E ) = ini_76reg_7E;
	*( ( unsigned char *)SED1376REG_80 ) = ini_76reg_80;
	*( ( unsigned char *)SED1376REG_81 ) = ini_76reg_81;
	*( ( unsigned char *)SED1376REG_84 ) = ini_76reg_84;
	*( ( unsigned char *)SED1376REG_85 ) = ini_76reg_85;
	*( ( unsigned char *)SED1376REG_88 ) = ini_76reg_88;
	*( ( unsigned char *)SED1376REG_89 ) = ini_76reg_89;
	*( ( unsigned char *)SED1376REG_8C ) = ini_76reg_8C;
	*( ( unsigned char *)SED1376REG_8D ) = ini_76reg_8D;
	*( ( unsigned char *)SED1376REG_90 ) = ini_76reg_90;
	*( ( unsigned char *)SED1376REG_91 ) = ini_76reg_91;
	*( ( unsigned char *)SED1376REG_A1 ) = ini_76reg_A1;
	*( ( unsigned char *)SED1376REG_A3 ) = ini_76reg_A3;
	*( ( unsigned char *)SED1376REG_A4 ) = ini_76reg_A4;
	*( ( unsigned char *)SED1376REG_A5 ) = ini_76reg_A5;
	*( ( unsigned char *)SED1376REG_A8 ) = ini_76reg_A8;
	*( ( unsigned char *)SED1376REG_A9 ) = ini_76reg_A9;
	*( ( unsigned char *)SED1376REG_AC ) = ini_76reg_AC;
	*( ( unsigned char *)SED1376REG_B0 ) = ini_76reg_B0;
	*( ( unsigned char *)SED1376REG_B1 ) = ini_76reg_B1;
	*( ( unsigned char *)SED1376REG_B2 ) = ini_76reg_B2;
	*( ( unsigned char *)SED1376REG_B3 ) = ini_76reg_B3;

	for( i=0; i<0x10; i++ )
	{
		*( ( unsigned char * ) SED1376REG_09 ) = LUT_DAT[i];
		*( ( unsigned char * ) SED1376REG_0B ) = i;
	}

	// GPIO5-6 set to be writeable
	*(unsigned short *)MR2GIUDIR |= 0x0060;
	// GPIO6 set to high, blacklight on.
	*(unsigned short *)MR2GIUPIOD |= 0x0040;

	*( ( unsigned char * )SED1376REG_A0 ) = 0x00;
	*( ( unsigned char * )SED1376REG_AD ) = 0x80;

	extlcdc_76ram();

// For debugging
#if 0
	j = 0;
	for( i = VRAM_start; i < (VRAM_end_ALPS+1); i+=1 )
	{
		if ( i%2 )
			mem_dat = 0xff;
		else
			mem_dat = 0x00;

		*( ( unsigned char * ) i ) = mem_dat;
		j++;
	}
#endif
        /* jackal add for contrast */
        pm_contrast_resume();
   
	// GPIO5 set to high, lcd on.
	*(unsigned short *)MR2GIUPIOD |= 0x0020;
        //contrast(15);
   
#if 0 /* 20020801 , jackal mask */
	init_timer(&lcd_timer);
	lcd_timer.function = ( void (*)(unsigned long) )blank_screen;
	mod_timer(&lcd_timer, jiffies + blankinterval);
#endif
	return VRAM_start;
}

static void contrast(int level)
{
  int i;

  if ( cur_level == -1 )
  {
    for ( i = 0 ; i < 32 ; i++ )
      contrast_down();

    cur_level = 0;
  }

  if ( level < 0 || level >= 32 || level == cur_level )
    return;

  if ( level > cur_level )
  {
    for ( i = 0 ; i < level-cur_level ; i++ )
      contrast_up();
  }
  else
  {
    for ( i = 0 ; i < cur_level-level ; i++ )
      contrast_down();
  }

  cur_level = level;
}


static void contrast_up(void)
{
  extlcdc_76reg();/*command*/

#ifdef CIH
  contrast_default();
#else
  contrast_set_init(Up);
#endif

#ifdef CIH
  // sed1376 gpio0 is from high to low
  *((unsigned char *)SED1376REG_AC) |= 0x01;
  *((unsigned char *)SED1376REG_AC) &= 0xfe;
  udelay(4);

  // sed1376 gpio2 is set to high, contrast up
  *((unsigned char *)SED1376REG_AC) |= 0x04;

  // sed1376 gpio1 is from low to high, then to low
  // occur one plus
  *((unsigned char *)SED1376REG_AC) &= 0xfd;
  *((unsigned char *)SED1376REG_AC) |= 0x02;
  udelay(12);
  *((unsigned char *)SED1376REG_AC) &= 0xfd;
  udelay(12);

  contrast_default();
#else
*((unsigned char *)SED1376REG_AC) &= 0xfe;/* enable xCS_GPIO0 */
INC_plus_GPIO1(1);
*((unsigned char *)SED1376REG_AC) |= 0x01;/* disable xCS_GPIO0 */

#endif

  extlcdc_76ram();
}

static void contrast_down(void)
{
  extlcdc_76reg(); /*command*/

#ifdef CIH
  contrast_default();
#else
 contrast_set_init(Down);
#endif

#ifdef CIH
  // sed1376 gpio0(xCS) form height to low
  *((unsigned char *)SED1376REG_AC) |= 0x01;
  *((unsigned char *)SED1376REG_AC) &= 0xfe;
  udelay(4);

  // sed1376 gpio2 is set to low, contrast down
  *((unsigned char *)SED1376REG_AC) &= 0xfb;

  // sed1376 gpio1 is from low to high, then to low
  // occur one plus
  *((unsigned char *)SED1376REG_AC) &= 0xfd;
  *((unsigned char *)SED1376REG_AC) |= 0x02;
  udelay(12);
  *((unsigned char *)SED1376REG_AC) &= 0xfd;
  udelay(12);
  contrast_default();
#else
*((unsigned char *)SED1376REG_AC) &= 0xfe;/* enable xCS_GPIO0 */
INC_plus_GPIO1(1);
*((unsigned char *)SED1376REG_AC) |= 0x01;/* disable xCS_GPIO0 */
#endif

  extlcdc_76ram();
}

#ifdef CIH
static void contrast_default(void)
{
  *((unsigned char *)SED1376REG_AC) |= 0x01;
  udelay(2);
  *((unsigned char *)SED1376REG_AC) &= 0xfd;
  udelay(2);
  *((unsigned char *)SED1376REG_AC) &= 0xfb;
  udelay(1000);
}
#else
static void contrast_set_init(count Up_Down)
{
  *((unsigned char *)SED1376REG_AC) |= 0x01; /* GPIO0(xCS)  = H */
  *((unsigned char *)SED1376REG_AC) |= 0x02; /* GPIO1(xINC) = H */

  if(Up_Down == Up)
  *((unsigned char *)SED1376REG_AC) |= 0x04; /* GPIO2(U/xD) = H */
  else
  *((unsigned char *)SED1376REG_AC) &= 0xfb; /* GPIO2(U/xD) = L */
}

static void INC_plus_GPIO1(int plus)
{
if(plus == 0)return;
if( *((unsigned char *)SED1376REG_AC) & 0x02 == 0) /* if  GPIO1(xINC) == L */
  *((unsigned char *)SED1376REG_AC) |= 0x02;	   /* set GPIO1(xINC) = H */

  while(1)
  {
    udelay(2);
    *((unsigned char *)SED1376REG_AC) &= 0xfd; /* set GPIO1(xINC) = L */
    udelay(2);
    *((unsigned char *)SED1376REG_AC) |= 0x02; /* set GPIO1(xINC) = H */

    plus--;
    if(plus == 0)break;
  }

}
#endif

static void blank_screen(void)
{ 
  if(Mask)return;
  blank = 1;
#if 0   
  del_timer(&lcd_timer);
#endif   
  lcd_screen_off();
}

#if 0 /* 20020801, jackal mask */
int pm_access_device(void)
{
    int i = 0;
  
    if ( blank )
    {
        blank = 0;
        lcd_screen_on();
        i = 1;
    }

//    del_timer(&lcd_timer);
//    mod_timer(&lcd_timer, jiffies + blankinterval);
    
    return i;
}
#endif

void lcd_screen_on(void)
{
   // GPIO5-6 set to be writeable
   *(unsigned short *)MR2GIUDIR |= 0x0060;
   
   // GPIO6 set to high, blacklight on.
   *(unsigned short *)MR2GIUPIOD |= 0x0040;
   
   // GPIO5 set to high, lcd on.
   *(unsigned short *)MR2GIUPIOD |= 0x0020;
}

void lcd_screen_off(void)
{
   // GPIO5-6 set to be writeable
   *(unsigned short *)MR2GIUDIR |= 0x0060;
   
   // GPIO6 set to low, blacklight off.
   *(unsigned short *)MR2GIUPIOD &= 0xffbf;
   
   // GPIO5 set to low, lcd off.
   *(unsigned short *)MR2GIUPIOD &= 0xffdf;
}

void set_blank_interval(int interval)
{
 blankinterval = interval*60*HZ; 
}

int read_lcd_off_interval(void)
{
    return blankinterval;
}

void set_pow_low_interval(int interval)
{
    blankinterval = interval*HZ;
}
 
void set_blank_mask(int mask)
{
    Mask = mask;
}
void set_blank(int backoff)
{
    blank=backoff;
}
int read_blank_mask(void)
{
    return Mask;
}

//#ifdef CONFIG_PM_SUSPEND_WAKEUP
#ifdef CONFIG_PM_SLEEP

void pm_lcd_on(void)
{
   /* gpio5 -> lcd 
    * gpio6 -> backlight
    */
   *(unsigned short *)MR2GIUDIR  |= 0x0020;
   udelay(100);
//   *(unsigned short *)MR2GIUPIOD |= 0x0040;
//   udelay(100);
   
   extlcdc_76reg();
   *(unsigned char *)SED1376REG_A0 = 0x00;	   
   *(unsigned char *)SED1376REG_AD = 0x80;
   
   *(unsigned short *)MR2GIUPIOD |= 0x0020;
   udelay(100);
   
   extlcdc_76ram();
}

void pm_lcd_off(void)
{
   *(unsigned short *)MR2GIUPIOD ^= 0x0020;
   udelay(100);
   
   extlcdc_76reg();
   *((unsigned char *)SED1376REG_AD) = 0x00;
   mdelay(200);
   
   *((unsigned char *)SED1376REG_A0) = 0x01;
   udelay(100);
//   *(unsigned short *)MR2GIUPIOD ^= 0x0040;
//   udelay(100);
   
   extlcdc_76ram();
   
}


void pm_sed1376_display_init( void )
{
	
   extlcdc_76reg();											/* M/R#[q̐؂ւ */
   
   *( ( unsigned char *)SED1376REG_04 ) = ini_76reg_04;
   *( ( unsigned char *)SED1376REG_05 ) = ini_76reg_05;
   
   *( ( unsigned char *)SED1376REG_10 ) = ini_76reg_10;
   *( ( unsigned char *)SED1376REG_11 ) = ini_76reg_11;		/* MOD[gWX^ */
   *( ( unsigned char *)SED1376REG_12 ) = ini_76reg_12;		/* SԃWX^ */
   *( ( unsigned char *)SED1376REG_14 ) = ini_76reg_14;		/* \ԃWX^ */
   *( ( unsigned char *)SED1376REG_16 ) = ini_76reg_16;		/* \ԊJnʒuWX^ 0 */
   *( ( unsigned char *)SED1376REG_17 ) = ini_76reg_17;		/* \ԊJnʒuWX^ 1 */
   *( ( unsigned char *)SED1376REG_18 ) = ini_76reg_18;		/* SԃWX^ 0 */
   *( ( unsigned char *)SED1376REG_19 ) = ini_76reg_19;		/* SԃWX^ 1 */
   *( ( unsigned char *)SED1376REG_1C ) = ini_76reg_1C;		/* \ԃWX^ 0 */
   *( ( unsigned char *)SED1376REG_1D ) = ini_76reg_1D;		/* \ԃWX^ 1 */
   *( ( unsigned char *)SED1376REG_1E ) = ini_76reg_1E;		/* \ԊJnʒuWX^ 0 */
   *( ( unsigned char *)SED1376REG_1F ) = ini_76reg_1F;		/* \ԊJnʒuWX^ 1 */
   *( ( unsigned char *)SED1376REG_20 ) = ini_76reg_20;		/* FPLINEpXWX^ */
   *( ( unsigned char *)SED1376REG_22 ) = ini_76reg_22;		/* FPLINEpXJnʒuWX^ 0 */
   *( ( unsigned char *)SED1376REG_23 ) = ini_76reg_23;		/* FPLINEpXJnʒuWX^ 1 */
   *( ( unsigned char *)SED1376REG_24 ) = ini_76reg_24;		/* FPFRAMEpXWX^ */
   *( ( unsigned char *)SED1376REG_26 ) = ini_76reg_26;		/* FPPFRAMEpXJnʒuWX^ 0 */
   *( ( unsigned char *)SED1376REG_27 ) = ini_76reg_27;		/* FPPFRAMEpXJnʒuWX^ 1 */
   
   *( ( unsigned char *)SED1376REG_70 ) = ini_76reg_70;		/* \[hWX^ */
   *( ( unsigned char *)SED1376REG_71 ) = ini_76reg_71;		/* GtFNgWX^ */
   *( ( unsigned char *)SED1376REG_74 ) = ini_76reg_74;		/* CEBhE\JnAhXWX^ 0 */
   *( ( unsigned char *)SED1376REG_75 ) = ini_76reg_75;		/* CEBhE\JnAhXWX^ 1 */
   *( ( unsigned char *)SED1376REG_76 ) = ini_76reg_76;		/* CEBhE\JnAhXWX^ 2 */
   *( ( unsigned char *)SED1376REG_78 ) = ini_76reg_78;		/* CEBhECAhXItZbgWX^ 0 */
   *( ( unsigned char *)SED1376REG_79 ) = ini_76reg_79;		/* CEBhECAhXItZbgWX^ 1 */
   *( ( unsigned char *)SED1376REG_7C ) = ini_76reg_7C;		/* TuEBhE\JnAhXWX^ 0 */
   *( ( unsigned char *)SED1376REG_7D ) = ini_76reg_7D;		/* TuEBhE\JnAhXWX^ 1 */
   *( ( unsigned char *)SED1376REG_7E ) = ini_76reg_7E;		/* TuEBhE\JnAhXWX^ 2 */
   *( ( unsigned char *)SED1376REG_80 ) = ini_76reg_80;		/* TuEBhECAhXItZbgWX^ 0 */
   *( ( unsigned char *)SED1376REG_81 ) = ini_76reg_81;		/* TuEBhECAhXItZbgWX^ 1 */
   *( ( unsigned char *)SED1376REG_84 ) = ini_76reg_84;		/* TuEBhE XJnʒuWX^ 0 */
   *( ( unsigned char *)SED1376REG_85 ) = ini_76reg_85;		/* TuEBhE XJnʒuWX^ 1 */
   *( ( unsigned char *)SED1376REG_88 ) = ini_76reg_88;		/* TuEBhE YJnʒuWX^ 0 */
   *( ( unsigned char *)SED1376REG_89 ) = ini_76reg_89;		/* TuEBhE YJnʒuWX^ 1 */
   *( ( unsigned char *)SED1376REG_8C ) = ini_76reg_8C;		/* TuEBhE XIʒuWX^ 0 */
   *( ( unsigned char *)SED1376REG_8D ) = ini_76reg_8D;		/* TuEBhE XIʒuWX^ 1 */
   *( ( unsigned char *)SED1376REG_90 ) = ini_76reg_90;		/* TuEBhE YIʒuWX^ 0 */
   *( ( unsigned char *)SED1376REG_91 ) = ini_76reg_91;		/* TuEBhE XIʒuWX^ 1 */

   *( ( unsigned char *)SED1376REG_A1 ) = ini_76reg_A1;		/* \ */

   *( ( unsigned char *)SED1376REG_A3 ) = ini_76reg_A3;		/* \ */
   *( ( unsigned char *)SED1376REG_A4 ) = ini_76reg_A4;		/* XNb`pbhWX^ 0 */
   *( ( unsigned char *)SED1376REG_A5 ) = ini_76reg_A5;		/* XNb`pbhWX^ 1 */
   *( ( unsigned char *)SED1376REG_A8 ) = ini_76reg_A8;		/* ėpIO[qݒ背WX^ 0 */
   *( ( unsigned char *)SED1376REG_A9 ) = ini_76reg_A9;		/* ėpIO[qݒ背WX^ 1 */
   *( ( unsigned char *)SED1376REG_AC ) = ini_76reg_AC;		/* ėpIO[qXe[^X^䃌WX^ 0 */

   *( ( unsigned char *)SED1376REG_B0 ) = ini_76reg_B0;		/* PWMNbN^CVpX䃌WX^ */
   *( ( unsigned char *)SED1376REG_B1 ) = ini_76reg_B1;		/* PWMNbN^CVpXݒ背WX^ */
   *( ( unsigned char *)SED1376REG_B2 ) = ini_76reg_B2;		/* CVpXo[XgWX^ */
   *( ( unsigned char *)SED1376REG_B3 ) = ini_76reg_B3;		/* PWMOUTf[eBTCNWX^1 */
}

void pm_sed1376_set_luttbl( void )
{
   unsigned short i = 0;
	
   for( i=0; i<0x10; i++ ){
      *( ( unsigned char * ) SED1376REG_09 ) = LUT_DAT[i];	
      *( ( unsigned char * ) SED1376REG_0B ) = i;				
   }
	
}

void pm_lcd_callback(int rqst)
{
   unsigned long i,j;
   static unsigned char vram[VRAM_end - VRAM_start + 1];
   
   switch(rqst){
    case 1:
      extlcdc_76reg();
      *( ( unsigned char *)SED1376REG_04 ) = 0x00;
      *( ( unsigned char *)SED1376REG_A0 ) = 0x00;
      
      extlcdc_76ram();      
      j = 0;
      for ( i = VRAM_start; i < (VRAM_end + 1); i+=1 ){
	 vram[j] = *( ( unsigned char * ) i );
	 j++;
      }
      break;
      
    case 2:
      extlcdc_76reg();
      *( ( unsigned char *)SED1376REG_04 ) = 0x00;
      *( ( unsigned char *)SED1376REG_A0 ) = 0x00;
      
      extlcdc_76ram();            
      j = 0;
      for ( i = VRAM_start; i < (VRAM_end + 1); i+=1 ){
	 *( ( unsigned char * ) i ) = vram[j];
	 j++;
      }
      
      break;
   }
   
}

void pm_sed1376_init(void)
{
   unsigned char mem_dat;
   unsigned long i;

   extern void pm_contrast_restore(void);
   
   *(unsigned short *)MR2GIUDIR  |= 0x0080;
   *(unsigned short *)MR2GIUPIOD |= 0x0080;
   
   pm_sed1376_display_init();
   pm_sed1376_set_luttbl();   
   pm_lcd_callback(2);
   extlcdc_76ram();
   
   pm_contrast_resume();
}
#endif


#if 0
EXPORT_SYMBOL(pm_access_device);
#endif

