/* 
 * Touch Panel Data Structures
 * by Michael Klar, wyldfier@iname.com
 *
 * Created for the touch panel interface built into the NEC VR41xx family of
 * CPUs, but should be applicable to other touch panel hardware
 */

#ifndef _LINUX_TPANEL_H
#define _LINUX_TPANEL_H

#include <linux/ioctl.h>

/*
 * The scan interval and settling time can be set via IOCTL, but the defualt
 * of .01 sec and 480us should suffice for most applications.  The argument to
 * these IOCTL calls is a struct of the values in microseconds, but note that
 * the hardware may have a less precise interval.  If so, the value will be
 * rounded and the actual value returned back.
 */

struct scanparam {
	unsigned int	interval;
	unsigned int	settletime;
};

struct calibrateparm {
        int raw_x_left;
        int raw_x_right;
        int raw_x_width;
        int raw_y_top;
        int raw_y_down;
        int raw_y_width;
        unsigned char xy_exchanged_x_inverted;  // Road-specific
};


#define TPGETSCANPARM _IOR( 0xB0, 0x00, struct scanparam )
#define TPSETSCANPARM _IOW( 0xB0, 0x01, struct scanparam )
#define TPGETCALIBRATEDPARM _IOR( 0xB0, 0x02, struct calibrateparm)
#define TPSETCALIBRATEDPARM _IOW( 0xB0, 0x03, struct calibrateparm)
#define PRESSURE_LOW_BONND 850  // fileter low-pressure data limit

#endif /* _LINUX_TPANEL_H */
